@@ -7,16 +7,116 @@ from eomaps.callbacks import (
 from types import SimpleNamespace
 
 from functools import update_wrapper, partial, wraps
-from collections import defaultdict
 import matplotlib.pyplot as plt
 
 from pyproj import Transformer
 
 import numpy as np
 
+gpd = None
+
+
+def _register_geopandas():
+    global gpd
+    try:
+        import geopandas as gpd
+    except ImportError:
+        return False
+
+    return True
+
+
+class _gpd_picker:
+    # a collection of pick-methods for geopandas.GeoDataFrames
+    def __init__(self, gdf, val_key, pick_method):
+        self.gdf = gdf
+        self.val_key = val_key
+        self.pick_method = pick_method
+
+    def get_picker(self):
+        assert _register_geopandas(), (
+            "EOmaps: Missing dependency `geopandas`!\n"
+            + "please install '(conda install -c conda-forge geopandas)'"
+            + "to make geopandas GeoDataFrames pickable."
+        )
+
+        if self.pick_method == "contains":
+            return self._contains_picker
+        elif self.pick_method == "centroids":
+            from scipy.spatial import cKDTree
+
+            self.tree = cKDTree(
+                list(map(lambda x: (x.x, x.y), self.gdf.geometry.centroid))
+            )
+            return self._centroids_picker
+        else:
+            raise TypeError(
+                f"EOmaps: {self.pick_method} is not a valid " "pick_method!"
+            )
+
+    def _contains_picker(self, artist, mouseevent):
+        try:
+            query = getattr(self.gdf, "contains")(
+                gpd.points_from_xy(
+                    np.atleast_1d(mouseevent.xdata),
+                    np.atleast_1d(mouseevent.ydata),
+                )[0]
+            )
+
+            if query.any():
+
+                ID = self.gdf.index[query][0]
+                ind = query.values.nonzero()[0][0]
+
+                if self.val_key:
+                    val = self.gdf[query][self.val_key].iloc[0]
+                else:
+                    val = None
+
+                if artist.get_array() is not None:
+                    val_numeric = artist.norm(artist.get_array()[ind])
+                    val_color = artist.cmap(val_numeric)
+                else:
+                    val_numeric = None
+                    val_color = None
+
+                return True, dict(
+                    ID=ID,
+                    ind=ind,
+                    val=val,
+                    val_color=val_color,
+                    pos=(mouseevent.xdata, mouseevent.ydata),
+                )
+            else:
+                return False, dict()
+        except Exception:
+            return False, dict()
+
+    def _centroids_picker(self, artist, mouseevent):
+        try:
+            dist, ind = self.tree.query((mouseevent.xdata, mouseevent.ydata), 1)
+            ID = self.gdf.index[ind]
+
+            if self.val_key is not None:
+                val = self.gdf.iloc[ind][self.val_key]
+            else:
+                val = None
+
+            pos = self.tree.data[ind].tolist()
+            try:
+                val_numeric = artist.norm(artist.get_array()[ID])
+                val_color = artist.cmap(val_numeric)
+            except Exception:
+                val_color = None
+
+            return True, dict(ID=ID, pos=pos, val=val, ind=ind, val_color=val_color)
+
+        except Exception:
+            return False, dict()
+
 
 class _cb_container(object):
-    """base-class for callback containers"""
+    """Base-class for callback containers."""
 
     def __init__(self, m, cb_class=None, method="click", tmp_artists=None):
         self._m = m
@@ -38,15 +138,12 @@ class _cb_container(object):
         self._event = None
 
     def _getobj(self, m):
-        """get the equivalent callback container on another maps object"""
+        """Get the equivalent callback container on another maps object."""
         return getattr(m.cb, self._method, None)
 
     @property
     def _objs(self):
-        """
-        get the callback-container objects associated with the axes that
-        the event belonged to
-        """
+        """Get the callback-container objects associated with the event-axes."""
         # Note: it is possible that more than 1 Maps objects are
         # assigned to the same axis!
         objs = []
@@ -75,7 +172,7 @@ class _cb_container(object):
     def _clear_temporary_artists(self):
         while len(self._temporary_artists) > 0:
             art = self._temporary_artists.pop(-1)
-            self._m.BM._artists_to_clear[self._method].append(art)
+            self._m.BM._artists_to_clear.setdefault(self._method, []).append(art)
 
     def _sort_cbs(self, cbs):
         if not cbs:
@@ -94,7 +191,7 @@ class _cb_container(object):
 
     def forward_events(self, *args):
         """
-        Forward callback-events from this Maps-object to other Maps-objects
+        Forward callback-events from this Maps-object to other Maps-objects.
 
         (e.g. share events one-way)
 
@@ -108,7 +205,7 @@ class _cb_container(object):
 
     def share_events(self, *args):
         """
-        Share callback-events between this Maps-object and all other Maps-objects
+        Share callback-events between this Maps-object and all other Maps-objects.
 
         (e.g. share events both ways)
 
@@ -117,7 +214,6 @@ class _cb_container(object):
         args : eomaps.Maps
             The Maps-objects that should execute the callback.
         """
-
         for m1 in (self._m, *args):
             for m2 in (self._m, *args):
                 if m1 is not m2:
@@ -128,7 +224,7 @@ class _cb_container(object):
 
     def add_temporary_artist(self, artist):
         """
-        Make an artist temporary (remove it from the map at the next event)
+        Make an artist temporary (remove it from the map at the next event).
 
         Parameters
         ----------
@@ -140,7 +236,7 @@ class _cb_container(object):
 
     def _execute_cb(self, layer):
         """
-        Get bool if a callback assigned on "layer" should be executed
+        Get bool if a callback assigned on "layer" should be executed.
 
         - True if the callback is assigned to the "all" layer
         - True if the corresponding layer is currently active
@@ -274,7 +370,7 @@ class _click_container(_cb_container):
 
         def __call__(self, f, double_click=False, button=None, modifier=None, **kwargs):
             """
-            add a custom callback-function to the map
+            Add a custom callback-function to the map.
 
             Parameters
             ----------
@@ -328,15 +424,9 @@ class _click_container(_cb_container):
                 the ID of the attached callback
 
             """
-
             if button is None:
                 button = self._parent._default_button
 
-            if self._parent._method == "pick":
-                assert (
-                    self._parent._m.coll is not None
-                ), "you can only attach pick-callbacks after calling `plot_map()`!"
-
             return self._parent._add_callback(
                 callback=f,
                 double_click=double_click,
@@ -346,15 +436,13 @@ class _click_container(_cb_container):
             )
 
     class _get:
-        """
-        Accessor for objects generated/retrieved by callbacks.
-        """
+        """Accessor for objects generated/retrieved by callbacks."""
 
         def __init__(self, parent):
             self.m = parent._m
             self.cb = parent._cb
 
-            self.cbs = defaultdict(lambda: defaultdict(dict))
+            self.cbs = dict()
 
         @property
         def picked_object(self):
@@ -398,7 +486,7 @@ class _click_container(_cb_container):
 
     def _parse_cid(self, cid):
         """
-        get the
+        Parse a callbac-id.
 
         Parameters
         ----------
@@ -499,12 +587,32 @@ class _click_container(_cb_container):
         >>> m.cb.click.set_sticky_modifiers("1")
 
         """
-
         self._sticky_modifiers = list(map(str, args))
 
         if self._method == "click":
             self._m.cb._click_move._sticky_modifiers = args
 
+    def _init_picker(self):
+        assert (
+            self._m.coll is not None
+        ), "you can only attach pick-callbacks after calling `plot_map()`!"
+
+        try:
+            # Lazily make a plotted dataset pickable a
+            if getattr(self._m, "tree", None) is None:
+                from .helpers import searchtree
+
+                self._m.tree = searchtree(m=self._m._proxy(self._m))
+                self._m.cb.pick._set_artist(self._m.coll)
+                self._m.cb.pick._init_cbs()
+                self._m.cb._methods.add("pick")
+        except Exception as ex:
+            print(
+                "EOmaps: There was an error while trying to initialize "
+                "pick-callbacks!",
+                ex,
+            )
+
     def _add_callback(
         self,
         *args,
@@ -515,7 +623,7 @@ class _click_container(_cb_container):
         **kwargs,
     ):
         """
-        Attach a callback to the plot that will be executed if a pixel is clicked
+        Attach a callback to the plot that will be executed if a pixel is clicked.
 
         A list of pre-defined callbacks (accessible via `m.cb`) or customly defined
         functions can be used.
@@ -593,10 +701,11 @@ class _click_container(_cb_container):
             button = self._default_button
 
         if self._method == "pick":
-            assert self._m.coll is not None, (
-                "you can only attach pick-callbacks after plotting a dataset!"
-                + "... use `m.plot_map()` first."
-            )
+            if self._m.coll is None:
+                # lazily initialize the picker when the layer is fetched
+                self._m._data_manager._on_next_fetch.append(self._init_picker)
+            else:
+                self._init_picker()
 
         # attach "on_move" callbacks
         movecb_name = None
@@ -636,7 +745,7 @@ class _click_container(_cb_container):
         # check for modifiers
         button_modifier = f"{button}__{modifier}"
 
-        d = self.get.cbs[btn_key][button_modifier]
+        d = self.get.cbs.setdefault(btn_key, dict()).setdefault(button_modifier, dict())
 
         # get a unique name for the callback
         # name_idx__layer
@@ -673,7 +782,6 @@ class cb_click_container(_click_container):
 
     Methods
     -------
-
     attach : accessor for callbacks.
         Executing the functions will attach the associated callback to the map!
 
@@ -715,9 +823,9 @@ class cb_click_container(_click_container):
         clickdict = self._get_clickdict(event)
 
         if event.dblclick:
-            cbs = self.get.cbs["double"]
+            cbs = self.get.cbs.get("double", dict())
         else:
-            cbs = self.get.cbs["single"]
+            cbs = self.get.cbs.get("single", dict())
 
         # check for keypress-modifiers
         if (
@@ -744,7 +852,7 @@ class cb_click_container(_click_container):
                     cb(**clickdict)
 
     def _onrelease(self, event):
-        cbs = self.get.cbs["release"]
+        cbs = self.get.cbs.get("release", dict())
 
         # check for keypress-modifiers
         if (
@@ -779,12 +887,12 @@ class cb_click_container(_click_container):
 
     def _add_click_callback(self):
         def clickcb(event):
+            if not self._m.cb.get_execute_callbacks():
+                return
+
             try:
                 self._event = event
 
-                # ignore callbacks while dragging axes
-                if self._m._ignore_cb_events:
-                    return
                 # don't execute callbacks if a toolbar-action is active
                 if (
                     self._m.f.canvas.toolbar is not None
@@ -809,10 +917,10 @@ class cb_click_container(_click_container):
                 pass
 
         def releasecb(event):
+            if not self._m.cb.get_execute_callbacks():
+                return
+
             try:
-                # ignore callbacks while dragging axes
-                if self._m._ignore_cb_events:
-                    return
                 # don't execute callbacks if a toolbar-action is active
                 if (
                     self._m.f.canvas.toolbar is not None
@@ -894,7 +1002,6 @@ class cb_move_container(cb_click_container):
 
     Methods
     -------
-
     attach : accessor for callbacks.
         Executing the functions will attach the associated callback to the map!
 
@@ -931,6 +1038,9 @@ class cb_move_container(cb_click_container):
 
     def _add_move_callback(self):
         def movecb(event):
+            if not self._m.cb.get_execute_callbacks():
+                return
+
             try:
                 self._event = event
                 # only execute movecb if a mouse-button is holded down
@@ -952,9 +1062,6 @@ class cb_move_container(cb_click_container):
                             self._m.BM._clear_temp_artists(self._method)
                         return
 
-                # ignore callbacks while dragging axes
-                if self._m._ignore_cb_events:
-                    return
                 # don't execute callbacks if a toolbar-action is active
                 if (
                     self._m.f.canvas.toolbar is not None
@@ -963,7 +1070,13 @@ class cb_move_container(cb_click_container):
 
                 # execute onclick on the maps object that belongs to the clicked axis
                 # and forward the event to all forwarded maps-objects
+                update = False
                 for obj in self._objs:
+                    # check if there is a reason to update
+                    if update is False:
+                        if len(obj.get.attached_callbacks) > 0:
+                            update = True
+
                     # clear temporary artists before executing new callbacks to avoid
                     # having old artists around when callbacks are triggered again
                     obj._clear_temporary_artists()
@@ -973,7 +1086,15 @@ class cb_move_container(cb_click_container):
                     # forward callbacks to the connected maps-objects
                     obj._fwd_cb(event)
 
-                self._m.parent.BM.update(clear=self._method)
+                # only update if a callback is attached
+                # (to avoid constantly calling update)
+                if update:
+                    if self._button_down:
+                        if event.button:
+                            self._m.parent.BM.update(clear=self._method)
+                    else:
+                        self._m.parent.BM.update(clear=self._method)
+
             except ReferenceError:
                 pass
 
@@ -987,19 +1108,21 @@ class cb_move_container(cb_click_container):
 class cb_pick_container(_click_container):
     """
     Callbacks that select the nearest datapoint if you click on the map.
-    (you must plot a dataset with `m.plot_map()` first!)
 
     The event will search for the closest data-point and execute the callback
     with the properties (e.g. position , ID, value) of the selected point.
 
     Note
     ----
-    The threshold for the default picker can be set via the `pick_distance` argument.
-    Use `m.plot_map(pick_distance=20)` to specify the maximal distance (in pixels)
-    that is used to identify the closest datapoint.
+    To speed up identification of points for very large datasets, the search
+    is limited to points located inside a "search rectangle".
+    The side-length of this rectangle is determined in the plot-crs and can be
+    set via `m.cb.pick.set_props(search_radius=...)`.
+
+    The default is to use a side-length of 50 times the dataset-radius.
 
     Methods
-    --------
+    -------
     attach : accessor for callbacks.
         Executing the functions will attach the associated callback to the map!
 
@@ -1014,6 +1137,8 @@ class cb_pick_container(_click_container):
 
     set_sticky_modifiers : define keypress-modifiers that remain active after release
 
+    set_props : set the picking behaviour (e.g. number of points, search radius, etc.)
+
     """
 
     def __init__(self, picker_name="default", picker=None, *args, **kwargs):
@@ -1021,7 +1146,12 @@ class cb_pick_container(_click_container):
         self._cid_pick_event = dict()
         self._picker_name = picker_name
         self._artist = None
-        self._pick_distance = np.inf
+
+        self._n_ids = 1
+        self._consecutive_multipick = False
+        self._pick_relative_to_closest = True
+
+        self._search_radius = "50"
 
         if picker is None:
             self._picker = self._default_picker
@@ -1042,6 +1172,65 @@ class cb_pick_container(_click_container):
                 f"the picker {name} does not exist...", "use `m.cb.add_picker` first!"
             )
 
+    def set_props(
+        self,
+        n=None,
+        consecutive_pick=None,
+        pick_relative_to_closest=None,
+        search_radius=None,
+    ):
+        """
+        Set the picker-properties (number of picked points, max. search radius, etc.).
+
+        Only provided arguments will be updated!
+
+        Parameters
+        ----------
+        n : int, optional
+            The number of nearest neighbours to pick at each pick-event.
+            The default is 1.
+        consecutive_pick : bool, optional
+
+            - If True, pick-callbacks will be executed consecutively for each
+              picked datapoint.
+            - if False, pick-callbacks will get lists of all picked values
+              as input-arguments
+
+            The default is False.
+        pick_relative_to_closest : bool, optional
+            ONLY relevant if `n > 1`.
+
+            - If True: pick (n) nearest neighbours based on the center of the
+              closest identified datapoint
+            - If False: pick (n) nearest neighbours based on the click-position
+
+            The default is True.
+        search_radius : int, float, str or None optional
+            Set the radius of the area that is used to limit the number of
+            pixels when searching for nearest-neighbours.
+
+            if `int` or `float`:
+                The radius of the circle in units of the plot_crs
+            if `str:
+                A multiplication-factor for the estimated pixel-radius.
+                (e.g. a circle with (r=search_radius * m.shape.radius) is
+                used if possible and else np.inf is used.
+
+            The default is "50" (e.g. 50 times the pixel-radius).
+
+        """
+        if n is not None:
+            self._n_ids = n
+
+        if consecutive_pick is not None:
+            self._consecutive_multipick = consecutive_pick
+
+        if pick_relative_to_closest is not None:
+            self._pick_relative_to_closest = pick_relative_to_closest
+
+        if search_radius is not None:
+            self._search_radius = search_radius
+
     def _set_artist(self, artist):
         self._artist = artist
         self._artist.set_picker(self._picker)
@@ -1051,13 +1240,14 @@ class cb_pick_container(_click_container):
         self._add_pick_callback()
 
     def _default_picker(self, artist, event):
+
         # make sure that objects are only picked if we are on the right layer
         if not self._execute_cb(self._m.layer):
             return False, None
 
         try:
             # if no pick-callback is attached, don't identify the picked point
-            if len(self._m.cb.pick.get.cbs) == 0:
+            if len(self.get.cbs) == 0:
                 return False, None
         except ReferenceError:
             # in case we encounter a reference-error, remove the picker from the artist
@@ -1073,68 +1263,96 @@ class cb_pick_container(_click_container):
         if not np.isfinite((event.xdata, event.ydata)).all():
             return False, dict(ind=None, dblclick=event.dblclick, button=event.button)
 
+        # update the search-radius if necessary
+        # (do this here to allow setting a multiplier for the dataset-radius
+        # without having to plot it first!)
+        if self._search_radius != self._m.tree._search_radius:
+            self._m.tree.set_search_radius(self._search_radius)
+
         # find the closest point to the clicked pixel
-        dist, index = self._m.tree.query((event.xdata, event.ydata))
+        index = self._m.tree.query(
+            (event.xdata, event.ydata),
+            k=self._n_ids,
+            pick_relative_to_closest=self._pick_relative_to_closest,
+        )
 
         if index is not None:
+            pos = self._m._data_manager._get_xy_from_index(index, reprojected=True)
+            val = self._m._data_manager._get_val_from_index(index)
+            ID = self._m._data_manager._get_id_from_index(index)
+
+            try:
+                val_color = artist.cmap(artist.norm(val))
+            except Exception:
+                val_color = None
+
             return True, dict(
-                ind=index, dblclick=event.dblclick, button=event.button, dist=dist
+                dblclick=event.dblclick,
+                button=event.button,
+                ind=index,
+                ID=ID,
+                pos=pos,
+                val=val,
+                val_color=val_color,
             )
         else:
-            return True, dict(
-                ind=None, dblclick=event.dblclick, button=event.button, dist=dist
-            )
+            # do this to "unpick" previously picked datapoints if you click
+            # outside the data-extent
+            return True, dict(ind=None, dblclick=event.dblclick, button=event.button)
 
         return False, None
 
-    def _get_id(self, ind):
-        """
-        Identify the ID from a 1D list or range object or a numpy.ndarray
-        (to avoid very large numpy-arrays if no explicit IDs are provided)
-
-        Parameters
-        ----------
-        ind : int
-            The index of the flattened array.
-
-        Returns
-        -------
-        ID : any
-            The corresponding data-ID.
-        """
-
-        ids = self._m._props["ids"]
-        if isinstance(ids, (list, range)):
-            ID = ids[ind]
-        elif isinstance(ids, np.ndarray):
-            ID = ids.flat[ind]
-        else:
-            ID = "?"
-
-        return ID
-
     def _get_pickdict(self, event):
-        ind = event.ind
+        event_ind = event.ind
+        n_inds = len(np.atleast_1d(event_ind))
+        # mouseevent = event.mouseevent
+        noval = [None] * n_inds if n_inds > 1 else None
+
+        ID = getattr(event, "ID", noval)
+        pos = getattr(event, "pos", noval)
+        val = getattr(event, "val", noval)
+        ind = getattr(event, "ind", noval)
+        val_color = getattr(event, "val_color", noval)
+
         if ind is not None:
-            if self._m.coll is not None and event.artist is self._m.coll:
+            if self._consecutive_multipick is False:
+                # return all picked values as arrays
                 clickdict = dict(
-                    pos=self._m._get_xy_from_index(ind, reprojected=True),
-                    ID=self._get_id(ind),
-                    val=self._m._props["z_data"].flat[ind],
+                    ID=ID,  # convert IDs to numpy-arrays!
+                    pos=pos,
+                    val=val,
                     ind=ind,
+                    val_color=val_color,
                     picker_name=self._picker_name,
                 )
+
+                return clickdict
             else:
-                clickdict = dict(
-                    ID=getattr(event, "ID", None),
-                    pos=getattr(
-                        event, "pos", (event.mouseevent.xdata, event.mouseevent.ydata)
-                    ),
-                    val=getattr(event, "val", None),
-                    ind=getattr(event, "ind", None),
-                    picker_name=self._picker_name,
-                )
-            return clickdict
+                if n_inds > 1:
+                    clickdicts = []
+                    for i in range(n_inds):
+                        clickdict = dict(
+                            ID=ID[i],
+                            pos=(pos[0][i], pos[1][i]),
+                            val=val[i],
+                            ind=ind[i],
+                            val_color=val_color[i],
+                            picker_name=self._picker_name,
+                        )
+                        clickdicts.append(clickdict)
+                else:
+                    clickdicts = [
+                        dict(
+                            ID=ID,  # convert IDs to numpy-arrays!
+                            pos=pos,
+                            val=val,
+                            ind=ind,
+                            val_color=val_color,
+                            picker_name=self._picker_name,
+                        )
+                    ]
+
+                return clickdicts
 
     def _onpick(self, event):
         if event.artist is not self._artist:
@@ -1154,9 +1372,9 @@ class cb_pick_container(_click_container):
         clickdict = self._get_pickdict(event)
 
         if event.mouseevent.dblclick:
-            cbs = self.get.cbs["double"]
+            cbs = self.get.cbs.get("double", dict())
         else:
-            cbs = self.get.cbs["single"]
+            cbs = self.get.cbs.get("single", dict())
 
         # check for keypress-modifiers
         if (
@@ -1182,7 +1400,11 @@ class cb_pick_container(_click_container):
 
                 cb = bcbs[key]
                 if clickdict is not None:
-                    cb(**clickdict)
+                    if self._consecutive_multipick is False:
+                        cb(**clickdict)
+                    else:
+                        for c in clickdict:
+                            cb(**c)
 
     def _reset_cids(self):
         for method, cid in self._cid_pick_event.items():
@@ -1193,16 +1415,14 @@ class cb_pick_container(_click_container):
         # execute onpick and forward the event to all connected Maps-objects
 
         def pickcb(event):
+            if not self._m.cb.get_execute_callbacks():
+                return
 
             try:
                 # make sure pickcb is only executed if we are on the right layer
                 if not self._execute_cb(self._m.layer):
                     return
 
-                # check if we want to ignore callbacks
-                if self._m._ignore_cb_events:
-                    return
-
                 # don't execute callbacks if a toolbar-action is active
                 if (
                     self._m.f.canvas.toolbar is not None
@@ -1217,9 +1437,8 @@ class cb_pick_container(_click_container):
                 self._m.BM._clear_temp_artists(self._method)
 
                 self._event = event
-                # check if the artists has a custom picker assigned
-
-                self._m.BM._clear_temp_artists(self._method)
+                self._clear_temporary_artists()
+                # self._m.BM._clear_temp_artists(self._method)
 
                 # execute "_onpick" on the maps-object that belongs to the clicked axes
                 # and forward the event to all forwarded maps-objects
@@ -1229,9 +1448,6 @@ class cb_pick_container(_click_container):
 
                 self._m.BM._after_update_actions.append(self._clear_temporary_artists)
                 self._m.BM._clear_temp_artists(self._method)
-
-                # self._m.parent.BM.update(clear=self._method)
-                self._m.parent.BM._clear_temp_artists(self._method)
                 # don't update here... the click-callback will take care of it!
             except ReferenceError:
                 pass
@@ -1246,7 +1462,6 @@ class cb_pick_container(_click_container):
         # PickEvents have a .mouseevent property for the associated MouseEvent!
         if event.mouseevent.inaxes != self._m.ax:
             return
-
         for key, m in self._fwd_cbs.items():
             obj = self._getobj(m)
             obj._clear_temporary_artists()
@@ -1285,17 +1500,14 @@ class cb_pick_container(_click_container):
             )
 
             pick = obj._picker(obj._artist, dummymouseevent)
-
             if pick[1] is not None:
                 dummyevent.ID = pick[1].get("ID", None)
                 dummyevent.ind = pick[1].get("ind", None)
                 dummyevent.val = pick[1].get("val", None)
-
-                if "dist" in pick[1]:
-                    dummyevent.dist = pick[1].get("dist", None)
+                dummyevent.pos = pick[1].get("pos", None)
+                dummyevent.val_color = pick[1].get("val_color", None)
             else:
                 dummyevent.ind = None
-                dummyevent.dist = None
 
             obj._onpick(dummyevent)
 
@@ -1306,7 +1518,6 @@ class keypress_container(_cb_container):
 
     Methods
     -------
-
     attach : accessor for callbacks.
         Executing the functions will attach the associated callback to the map!
 
@@ -1342,10 +1553,10 @@ class keypress_container(_cb_container):
 
     def _initialize_callbacks(self):
         def _onpress(event):
-            try:
-                if self._m.parent._layout_editor._modifier_pressed:
-                    return
+            if not self._m.cb.get_execute_callbacks():
+                return
 
+            try:
                 self._event = event
 
                 # remember keypress event in case sticky modifiers are used for
@@ -1372,23 +1583,24 @@ class keypress_container(_cb_container):
                         print(f"EOmaps: sticky modifier: {k} ({', '.join(methods)})")
                         self._modifier = k
 
-                update = False
                 for obj in self._objs:
                     # only trigger callbacks on the right layer
                     if not self._execute_cb(obj._m.layer):
                         continue
                     if any(i in obj.get.cbs for i in (event.key, None)):
-                        update = True
                         # do this to allow deleting callbacks with a callback
                         # otherwise modifying a dict during iteration is problematic!
-                        cbs = {**obj.get.cbs[event.key], **obj.get.cbs[None]}
+                        cbs = {
+                            **obj.get.cbs.get(event.key, dict()),
+                            **obj.get.cbs.get(None, dict()),
+                        }
 
                         names = list(cbs)
                         for name in names:
                             if name in cbs:
                                 cbs[name](key=event.key)
-                # if update:
-                #     self._m.parent.BM.update(clear=self._method)
+
+                self._m.parent.BM.update(clear=self._method)
             except ReferenceError:
                 pass
 
@@ -1413,17 +1625,16 @@ class keypress_container(_cb_container):
 
         Examples
         --------
+        Attach a pre-defined callback:
 
-            Attach a pre-defined callback:
+        >>> m.cb.keypress.attach.switch_layer(layer=1, key="1")
 
-            >>> m.cb.keypress.attach.switch_layer(layer=1, key="1")
+        Attach a custom callback:
 
-            Attach a custom callback:
-
-            >>> def cb(**kwargs):
-            >>>     ... do something ...
-            >>>
-            >>> m.cb.keypress.attach(cb, key="3")
+        >>> def cb(**kwargs):
+        >>>     ... do something ...
+        >>>
+        >>> m.cb.keypress.attach(cb, key="3")
 
         """
 
@@ -1443,7 +1654,7 @@ class keypress_container(_cb_container):
 
         def __call__(self, f, key, **kwargs):
             """
-            Add a custom callback-function to the map
+            Add a custom callback-function to the map.
 
             Parameters
             ----------
@@ -1471,7 +1682,6 @@ class keypress_container(_cb_container):
                 the ID of the attached callback
 
             """
-
             if key is not None and not isinstance(key, str):
                 raise TypeError(
                     "EOmaps: The 'key' for keypress-callbacks must be a string!"
@@ -1484,7 +1694,7 @@ class keypress_container(_cb_container):
             self.m = parent._m
             self.cb = parent._cb
 
-            self.cbs = defaultdict(dict)
+            self.cbs = dict()
 
         @property
         def attached_callbacks(self):
@@ -1503,15 +1713,15 @@ class keypress_container(_cb_container):
 
     def remove(self, callback=None):
         """
-        remove an attached callback from the figure
+        Remove an attached callback from the figure.
 
         Parameters
         ----------
         callback : int, str or tuple
             if str: the name of the callback to remove
                     (`<function_name>_<count>__<layer>__<key>`)
-        """
 
+        """
         if callback is not None:
             name, layer, key = self._parse_cid(callback)
 
@@ -1534,7 +1744,7 @@ class keypress_container(_cb_container):
 
     def _add_callback(self, callback, key="x", **kwargs):
         """
-        Attach a callback to the plot that will be executed if a key is pressed
+        Attach a callback to the plot that will be executed if a key is pressed.
 
         A list of pre-defined callbacks (accessible via `m.cb`) or customly defined
         functions can be used.
@@ -1566,7 +1776,6 @@ class keypress_container(_cb_container):
             (to remove the callback, use `m.cb.remove(cbname)`)
 
         """
-
         if isinstance(callback, str):
             assert hasattr(self._cb, callback), (
                 f"The function '{callback}' does not exist as a pre-defined callback."
@@ -1575,7 +1784,7 @@ class keypress_container(_cb_container):
             )
             callback = getattr(self._cb, callback)
 
-        cbdict = self.get.cbs[key]
+        cbdict = self.get.cbs.setdefault(key, dict())
 
         # get a unique name for the callback
         ncb = [
@@ -1601,7 +1810,6 @@ class cb_container:
 
     Methods
     -------
-
     - **click** : Execute functions when clicking on the map
 
     - **pick** : Execute functions when you "pick" a pixel on the  map
@@ -1614,7 +1822,7 @@ class cb_container:
     def __init__(self, m):
         self._m = m
 
-        self._methods = ["click", "move", "keypress", "_click_move"]
+        self._methods = {"click", "move", "keypress", "_click_move"}
 
         self._click = cb_click_container(
             m=self._m,
@@ -1650,24 +1858,51 @@ class cb_container:
             method="keypress",
         )
 
+    def get_execute_callbacks(self):
+        """
+        Get if callbacks should be executed or not.
+
+        Returns
+        -------
+        bool
+            If True, callbacks are executed.
+
+        """
+        return self._m.parent._execute_callbacks
+
+    def execute_callbacks(self, val):
+        """
+        Activate / deactivate triggering callbacks.
+
+        Parameters
+        ----------
+        val : bool
+            If True, callbacks will be executed.
+        """
+        self._m.parent._execute_callbacks = val
+
     @property
     @wraps(cb_click_container)
     def click(self):
+        """Attach click callbacks."""
         return self._click
 
     @property
     @wraps(cb_move_container)
     def move(self):
+        """Attach move callbacks."""
         return self._move
 
     @property
     @wraps(cb_pick_container)
     def pick(self):
+        """Attach pick callbacks."""
         return self._pick
 
     @property
     @wraps(keypress_container)
     def keypress(self):
+        """Attach keypress callbacks."""
         return self._keypress
 
     def add_picker(self, name, artist, picker):
@@ -1735,7 +1970,7 @@ class cb_container:
 
         # add the picker method to the accessible cbs
         setattr(self._m.cb, new_pick._method, new_pick)
-        self._methods.append(new_pick._method)
+        self._methods.add(new_pick._method)
 
         return new_pick
 
@@ -1746,6 +1981,12 @@ class cb_container:
 
         self._remove_default_keymaps()
 
+    def _clear_callbacks(self):
+        # clear all callback containers
+        for method in self._methods:
+            obj = getattr(self, method)
+            obj.get.cbs.clear()
+
     def _reset_cids(self):
         # reset the callback functions (required to re-attach the callbacks
         # in case the figure is closed and re-initialized)
