@@ -60,7 +60,7 @@ class shapes(object):
 
     Attributes
     ----------
-    radius_estimation_range : int
+    _radius_estimation_range : int
         The number of datapoints to use for estimating the radius of a shape.
         (only relevant if the radius is not specified explicitly.)
         The default is 100000
@@ -80,7 +80,7 @@ class shapes(object):
 
     def __init__(self, m):
         self._m = m
-        self.radius_estimation_range = 100000
+        self._radius_estimation_range = 100000
 
     def _get(self, shape, **kwargs):
         shp = getattr(self, f"_{shape}")(self._m)
@@ -93,29 +93,40 @@ class shapes(object):
         if (isinstance(radius, str) and radius == "estimate") or radius is None:
             if m._estimated_radius is None:
                 # make sure props are defined otherwise we can't estimate the radius!
-                if not hasattr(m, "_props"):
-                    m._props = m._prepare_data()
+                if m._data_manager.x0 is None:
+                    m._data_manager.set_props(None)
 
                 print("EOmaps: estimating radius...")
                 radiusx, radiusy = shapes._estimate_radius(m, radius_crs)
 
                 if radiusx == radiusy:
-                    print(f"EOmaps: radius: {radiusx:.4f}")
+                    print(
+                        "EOmaps: radius:",
+                        np.format_float_scientific(radiusx, precision=4),
+                    )
                 else:
-                    print(f"EOmaps: radius: ({radiusx:.4f}, {radiusy:.4f})")
+                    print(
+                        "EOmaps: radius:" "(",
+                        np.format_float_scientific(radiusx, precision=4),
+                        ",",
+                        np.format_float_scientific(radiusy, precision=4),
+                        ")",
+                    )
                 radius = (radiusx, radiusy)
                 # remember estimated radius to avoid re-calculating it all the time
                 m._estimated_radius = (radiusx, radiusy)
             else:
                 radius = m._estimated_radius
         else:
-            if isinstance(radius, (list, np.ndarray)):
-                radiusx = radiusy = tuple(radius)
             # get manually specified radius (e.g. if radius != "estimate")
+            if isinstance(radius, (list, np.ndarray)):
+                radiusx = radiusy = np.asanyarray(radius).ravel()
             elif isinstance(radius, tuple):
                 radiusx, radiusy = radius
             elif isinstance(radius, (int, float, np.number)):
                 radiusx = radiusy = radius
+            else:
+                radiusx = radiusy = radius
 
             radius = (radiusx, radiusy)
         return radius
@@ -123,32 +134,26 @@ class shapes(object):
     @staticmethod
     def _estimate_radius(m, radius_crs, method=np.median):
 
-        if radius_crs == "in":
-            x, y = m._props["xorig"], m._props["yorig"]
-        elif radius_crs == "out":
-            x, y = m._props["x0"], m._props["y0"]
+        assert radius_crs in [
+            "in",
+            "out",
+        ], "radius can only be estimated if radius_crs is 'in' or 'out'!"
+
+        if m._data_manager.x0_1D is not None:
+            x, y = m._data_manager.x0_1D, m._data_manager.y0_1D
         else:
-            raise AssertionError(
-                "radius can only be estimated if radius_crs is 'in' or 'out'!"
-            )
+            if radius_crs == "in":
+                x, y = m._data_manager.xorig, m._data_manager.yorig
+            elif radius_crs == "out":
+                x, y = m._data_manager.x0, m._data_manager.y0
 
         radius = None
         # try to estimate radius for 2D datasets
-        if len(m._xshape) == 2 and len(m._yshape) == 2:
-            userange = int(np.sqrt(m.set_shape.radius_estimation_range))
+        if len(x.shape) == 2 and len(y.shape) == 2:
+            userange = int(np.sqrt(m.set_shape._radius_estimation_range))
 
-            radiusx = (
-                np.nanmedian(
-                    np.diff(x.reshape(m._xshape)[:userange, :userange], axis=1)
-                )
-                / 2
-            )
-            radiusy = (
-                np.nanmedian(
-                    np.diff(y.reshape(m._yshape)[:userange, :userange], axis=0)
-                )
-                / 2
-            )
+            radiusx = np.nanmedian(np.diff(x[:userange, :userange], axis=1)) / 2
+            radiusy = np.nanmedian(np.diff(y[:userange, :userange], axis=0)) / 2
 
             radius = (radiusx, radiusy)
 
@@ -156,20 +161,26 @@ class shapes(object):
                 radius = None
 
         # for 1D datasets (or if 2D radius-estimation fails), use the median distance
-        # of 3 neighbours of the first N datapoints (N=shape.radius_estimation_range)
+        # of 3 neighbours of the first N datapoints (N=shape._radius_estimation_range)
         if radius is None:
             from scipy.spatial import cKDTree
 
             # take care of 2D data with 1D coordinates
-            if m._1D2D:
-                userange = int(np.sqrt(m.set_shape.radius_estimation_range))
+            if m._data_manager.x0_1D is not None:
+                userange = int(np.sqrt(m.set_shape._radius_estimation_range))
                 x, y = np.meshgrid(x[:userange], y[:userange])
+                x, y = x.flat, y.flat
+            else:
+                x = x.flat[: m.set_shape._radius_estimation_range]
+                x = x[np.isfinite(x)]
+                y = y.flat[: m.set_shape._radius_estimation_range]
+                y = y[np.isfinite(y)]
 
             in_tree = cKDTree(
                 np.stack(
                     [
-                        x.flat[: m.set_shape.radius_estimation_range],
-                        y.flat[: m.set_shape.radius_estimation_range],
+                        x,
+                        y,
                     ],
                     axis=1,
                 ),
@@ -177,21 +188,42 @@ class shapes(object):
                 balanced_tree=False,
             )
 
-            dists, pts = in_tree.query(in_tree.data, 3)
+            dists, pts = in_tree.query(in_tree.data, min(len(in_tree.data), 3))
+            # consider only neighbors
+            # (the first entry is the search-point again!)
+            pts = pts[:, 1:]
+            # get the average distance between points having a distance > 0
+            d = np.abs(in_tree.data[:, np.newaxis] - in_tree.data[pts]).reshape(-1, 2)
 
-            radiusxy = method(dists) / 2
-
-            if not np.isfinite(radiusxy) or not (radiusxy > 0):
-                radius = None
+            use_dx = d[:, 0] > 0
+            use_dy = d[:, 1] > 0
+            if any(use_dx):
+                radiusx = method(d[:, 0][use_dx]) / 2
             else:
-                radius = (radiusxy, radiusxy)
+                radiusx = np.nan
+
+            if any(use_dy):
+                radiusy = method(d[:, 1][use_dy]) / 2
+            else:
+                radiusy = np.nan
+
+            rxOK = np.isfinite(radiusx) and (radiusx > 0)
+            ryOK = np.isfinite(radiusy) and (radiusy > 0)
+            if rxOK and ryOK:
+                radius = (radiusx, radiusy)
+            elif rxOK:
+                radius = (radiusx, radiusx)
+            elif ryOK:
+                radius = (radiusy, radiusy)
+            else:
+                radius = None
 
         assert radius is not None, (
             "EOmaps: Radius estimation failed... maybe there's something wrong with "
             "the provided coordinates? "
             "You can manually specify a radius with 'm.set_shape.<SHAPE>(radius=...)' "
             "or you can increase the number of datapoints used to estimate the radius "
-            "by increasing `m.set_shape.radius_estimation_range`."
+            "by increasing `m.set_shape._radius_estimation_range`."
         )
 
         return radius
@@ -211,7 +243,7 @@ class shapes(object):
         # ----------- manual color specifications
         # allow the synonyms "color", "fc" and "facecolor"
         color = None
-        for i in ["color", "fc", "facecolors"]:
+        for i in ["color", "fc", "facecolor"]:
             if color is None:
                 color = kwargs.pop(i, None)
                 if color is not None:
@@ -228,7 +260,7 @@ class shapes(object):
         if isinstance(color, (int, float, str, np.number)):
             # if a scalar is provided, broadcast it
             color = np.broadcast_to(color, mask.shape)
-        elif isinstance(color, tuple):
+        elif isinstance(color, (list, tuple)):
             if len(color) in [3, 4]:
                 if all(map(lambda i: isinstance(i, (int, float, np.number)), color)):
                     # check if a tuple of numbers is provided, and if so broadcast
@@ -247,7 +279,7 @@ class shapes(object):
 
     @staticmethod
     @lru_cache()
-    def get_transformer(in_crs, out_crs):
+    def _get_transformer(in_crs, out_crs):
         # cache transformers to avoid re-initialization for each feature
         t = Transformer.from_crs(in_crs, out_crs, always_xy=True)
         return t
@@ -257,6 +289,7 @@ class shapes(object):
 
         def __init__(self, m):
             self._m = m
+            self._n = None
 
         def __call__(self, radius=1000, n=None):
             """
@@ -264,8 +297,11 @@ class shapes(object):
 
             Parameters
             ----------
-            radius : float
+            radius : float or array-like
                 The radius of the circles in meters.
+
+                If you provide an array of sizes, each datapoint will be drawn with
+                the respective size!
             n : int or None
                 The number of intermediate points to calculate on the geodesic circle.
                 If None, 100 is used for < 10k pixels and 20 otherwise.
@@ -298,6 +334,8 @@ class shapes(object):
                         return 100
                     else:
                         return 20
+                else:
+                    return 20
             return self._n
 
         @n.setter
@@ -310,17 +348,7 @@ class shapes(object):
 
         @radius.setter
         def radius(self, val):
-            if not isinstance(val, (int, float)):
-                print("EOmaps: geod_circles only support a number as radius!")
-                if isinstance(val[0], (int, float)):
-                    print("EOmaps: ... using the mean")
-                    val = np.mean(val)
-                else:
-                    raise TypeError(
-                        f"EOmaps: '{val}' is not a valid radius for 'geod_circles'!"
-                    )
-
-            self._radius = val
+            self._radius = np.asanyarray(np.atleast_1d(val))
 
         @property
         def radius_crs(self):
@@ -355,7 +383,7 @@ class shapes(object):
             -------
             lons : array-like
                 the longitudes of the geodetic circle points.
-            lats : TYPE
+            lats : array-like
                 the latitudes of the geodetic circle points.
 
             """
@@ -364,7 +392,10 @@ class shapes(object):
             if isinstance(radius, (int, float)):
                 radius = np.full((size, n), radius)
             else:
-                radius = (np.broadcast_to(radius[:, None], (size, n)),)
+                if radius.size != lon.size:
+                    radius = np.broadcast_to(radius[:, None], (size, n))
+                else:
+                    radius = np.broadcast_to(radius.ravel()[:, None], (size, n))
 
             geod = self._m.crs_plot.get_geod()
             lons, lats, back_azim = geod.fwd(
@@ -383,9 +414,11 @@ class shapes(object):
             x, y = np.asarray(x), np.asarray(y)
 
             # transform from in-crs to lon/lat
-            radius_t = shapes.get_transformer(self._m.get_crs(crs), CRS.from_epsg(4326))
+            radius_t = shapes._get_transformer(
+                self._m.get_crs(crs), CRS.from_epsg(4326)
+            )
             # transform from lon/lat to the plot_crs
-            plot_t = shapes.get_transformer(
+            plot_t = shapes._get_transformer(
                 CRS.from_epsg(4326), CRS.from_user_input(self._m.crs_plot)
             )
 
@@ -402,8 +435,8 @@ class shapes(object):
                 ~xs.mask.any(axis=0)
                 & ~ys.mask.any(axis=0)
                 & ((dx / dy) < 10)
-                & (dx < radius * 50)
-                & (dy < radius * 50)
+                & (dx < np.max(radius) * 50)
+                & (dy < np.max(radius) * 50)
             )
 
             mask = np.broadcast_to(mask[:, None].T, lons.shape)
@@ -440,6 +473,65 @@ class shapes(object):
 
             return coll
 
+    class _scatter_points(object):
+        name = "scatter_points"
+
+        def __init__(self, m):
+            self._m = m
+            self._n = None
+
+        def __call__(self, size=None, marker=None):
+            """
+            Draw each datapoint as a shape with a size defined in points**2.
+
+            All arguments are forwarded to `m.ax.scatter()`.
+
+            Parameters
+            ----------
+            size : int, float, array-like or str, optional
+                The marker size in points**2.
+
+                If you provide an array of sizes, each datapoint will be drawn with
+                the respective size!
+            marker : str
+                The marker style. Can be either an instance of the class or the text
+                shorthand for a particular marker. Some examples are:
+
+                - `".", "o", "s", "<", ">", "^", "$A^2$"`
+
+                See matplotlib.markers for more information about marker styles.
+            """
+            from . import MapsGrid  # do this here to avoid circular imports!
+
+            for m in self._m if isinstance(self._m, MapsGrid) else [self._m]:
+                shape = self.__class__(m)
+                shape._size = size
+                shape._marker = marker
+                m._shape = shape
+
+        @property
+        def _initargs(self):
+            return dict(size=self._size, marker=self._marker)
+
+        @property
+        def radius(self):
+            radius = shapes._get_radius(self._m, "estimate", "in")
+            return radius
+
+        @property
+        def radius_crs(self):
+            return "in"
+
+        def get_coll(self, x, y, crs, **kwargs):
+            color_and_array = shapes._get_colors_and_array(
+                kwargs, np.full((x.size,), True)
+            )
+            color_and_array["c"] = color_and_array["array"]
+            coll = self._m.ax.scatter(
+                x, y, s=self._size, marker=self._marker, **color_and_array, **kwargs
+            )
+            return coll
+
     class _ellipses(object):
         name = "ellipses"
 
@@ -453,10 +545,13 @@ class shapes(object):
 
             Parameters
             ----------
-            radius : int, float, tuple or str, optional
+            radius : int, float, array-like or str, optional
                 The radius in x- and y- direction.
                 The default is "estimate" in which case the radius is attempted
                 to be estimated from the input-coordinates.
+
+                If you provide an array of sizes, each datapoint will be drawn with
+                the respective size!
             radius_crs : crs-specification, optional
                 The crs in which the dimensions are defined.
                 The default is "in".
@@ -488,6 +583,8 @@ class shapes(object):
                         return 100
                     else:
                         return 20
+                else:
+                    return 20
             return self._n
 
         @n.setter
@@ -501,7 +598,10 @@ class shapes(object):
 
         @radius.setter
         def radius(self, val):
-            self._radius = val
+            if isinstance(val, (list, np.ndarray)):
+                self._radius = np.asanyarray(val).ravel()
+            else:
+                self._radius = val
 
         def __repr__(self):
             try:
@@ -559,13 +659,12 @@ class shapes(object):
         def _get_ellipse_points(self, x, y, crs, radius, radius_crs="in", n=20):
             crs = self._m.get_crs(crs)
             radius_crs = self._m.get_crs(radius_crs)
-
             # transform from crs to the plot_crs
-            t_in_plot = shapes.get_transformer(crs, self._m.crs_plot)
+            t_in_plot = shapes._get_transformer(crs, self._m.crs_plot)
             # transform from crs to the radius_crs
-            t_in_radius = shapes.get_transformer(crs, radius_crs)
+            t_in_radius = shapes._get_transformer(crs, radius_crs)
             # transform from crs to the radius_crs
-            t_radius_plot = shapes.get_transformer(radius_crs, self._m.crs_plot)
+            t_radius_plot = shapes._get_transformer(radius_crs, self._m.crs_plot)
 
             if isinstance(radius, (int, float, np.number)):
                 rx, ry = radius, radius
@@ -579,8 +678,8 @@ class shapes(object):
                 xs, ys = self._calc_ellipse_points(
                     p[0],
                     p[1],
-                    np.full_like(x, rx, dtype=float),
-                    np.full_like(x, ry, dtype=float),
+                    np.broadcast_to(rx, x.shape).astype(float),
+                    np.broadcast_to(ry, y.shape).astype(float),
                     np.full_like(x, 0),
                     n=n,
                 )
@@ -592,8 +691,8 @@ class shapes(object):
                 xs, ys = self._calc_ellipse_points(
                     p[0],
                     p[1],
-                    np.full_like(x, rx, dtype=float),
-                    np.full_like(x, ry, dtype=float),
+                    np.broadcast_to(rx, x.shape).astype(float),
+                    np.broadcast_to(ry, y.shape).astype(float),
                     np.full_like(x, 0),
                     n=n,
                 )
@@ -622,8 +721,8 @@ class shapes(object):
 
                 return quadrants
 
-            t_in_lonlat = shapes.get_transformer(crs, 4326)
-            t_plot_lonlat = shapes.get_transformer(self._m.crs_plot, 4326)
+            t_in_lonlat = shapes._get_transformer(crs, 4326)
+            t_plot_lonlat = shapes._get_transformer(self._m.crs_plot, 4326)
 
             # transform the coordinates to lon/lat
             xp, _ = t_in_lonlat.transform(x, y)
@@ -662,8 +761,6 @@ class shapes(object):
                 if mask[i]
             )
 
-            # verts = np.ma.stack((xs, ys)).T.swapaxes(0, 1)
-
             color_and_array = shapes._get_colors_and_array(kwargs, mask)
             # remember masked points
             self._m._data_mask = mask
@@ -693,6 +790,9 @@ class shapes(object):
                 The radius in x- and y- direction.
                 The default is "estimate" in which case the radius is attempted
                 to be estimated from the input-coordinates.
+
+                If you provide an array of sizes, each datapoint will be drawn with
+                the respective size!
             radius_crs : crs-specification, optional
                 The crs in which the dimensions are defined.
                 The default is "in".
@@ -719,17 +819,8 @@ class shapes(object):
                 shape._radius = radius
                 shape.radius_crs = radius_crs
                 shape.mesh = mesh
+                shape.n = n
 
-                if mesh is True:
-                    if n is None:
-                        n = 1
-                    elif n > 1:
-                        warnings.warn(
-                            "EOmaps: rectangles with 'mesh=True' only supports n=1"
-                        )
-                    shape.n = 1
-                else:
-                    shape.n = n
                 m._shape = shape
 
         @property
@@ -759,7 +850,14 @@ class shapes(object):
 
         @n.setter
         def n(self, val):
-            self._n = val
+            if self.mesh is True:
+                if val is not None and val != 1:
+                    warnings.warn(
+                        "EOmaps: rectangles with 'mesh=True' only supports n=1"
+                    )
+                self._n = 1
+            else:
+                self._n = val
 
         @property
         def radius(self):
@@ -768,7 +866,10 @@ class shapes(object):
 
         @radius.setter
         def radius(self, val):
-            self._radius = val
+            if isinstance(val, (list, np.ndarray)):
+                self._radius = np.asanyarray(val).ravel()
+            else:
+                self._radius = val
 
         def __repr__(self):
             try:
@@ -790,13 +891,13 @@ class shapes(object):
             if radius_crs == crs:
                 in_crs = self._m.get_crs(crs)
                 # transform from crs to the plot_crs
-                t = shapes.get_transformer(
+                t = shapes._get_transformer(
                     CRS.from_user_input(in_crs), self._m.crs_plot
                 )
 
                 # make sure we do not transform out of bounds (if possible)
                 if in_crs.area_of_use is not None:
-                    transformer = shapes.get_transformer(in_crs.geodetic_crs, in_crs)
+                    transformer = shapes._get_transformer(in_crs.geodetic_crs, in_crs)
 
                     xmin, ymin, xmax, ymax = transformer.transform_bounds(
                         *in_crs.area_of_use.bounds
@@ -812,13 +913,13 @@ class shapes(object):
                 r_crs = self._m.get_crs(radius_crs)
 
                 # transform from crs to the radius_crs
-                t_in_radius = shapes.get_transformer(in_crs, r_crs)
+                t_in_radius = shapes._get_transformer(in_crs, r_crs)
                 # transform from radius_crs to the plot_crs
-                t = shapes.get_transformer(r_crs, self._m.crs_plot)
+                t = shapes._get_transformer(r_crs, self._m.crs_plot)
 
                 # make sure we do not transform out of bounds (if possible)
                 if r_crs.area_of_use is not None:
-                    transformer = shapes.get_transformer(r_crs.geodetic_crs, r_crs)
+                    transformer = shapes._get_transformer(r_crs.geodetic_crs, r_crs)
 
                     xmin, ymin, xmax, ymax = transformer.transform_bounds(
                         *r_crs.area_of_use.bounds
@@ -966,6 +1067,7 @@ class shapes(object):
 
         def __init__(self, m):
             self._m = m
+            self._mask_radius = None
 
         def __call__(self, masked=True, mask_radius=None):
             """
@@ -977,7 +1079,9 @@ class shapes(object):
                 Indicator if the voronoi-diagram should be masked or not
 
             mask_radius : float, optional
-                the radius used for masking the voronoi-diagram (in units of the plot-crs)
+                The radius used for masking the voronoi-diagram
+                (in units of the plot-crs)
+                The default is 4 times the estimated data-radius.
             """
             from . import MapsGrid  # do this here to avoid circular imports!
 
@@ -1003,7 +1107,11 @@ class shapes(object):
 
         @property
         def mask_radius(self):
-            return shapes._get_radius(self._m, self._mask_radius, "out")
+            r = shapes._get_radius(self._m, self._mask_radius, "out")
+            if self._mask_radius is None:
+                return (i * 4 for i in r)
+            else:
+                return r
 
         @mask_radius.setter
         def mask_radius(self, val):
@@ -1017,7 +1125,7 @@ class shapes(object):
                 raise ImportError("'scipy' is required for 'voronoi'!")
 
             # transform from crs to the plot_crs
-            t_in_plot = shapes.get_transformer(self._m.get_crs(crs), self._m.crs_plot)
+            t_in_plot = shapes._get_transformer(self._m.get_crs(crs), self._m.crs_plot)
 
             x0, y0 = t_in_plot.transform(x, y)
 
@@ -1094,6 +1202,7 @@ class shapes(object):
 
         def __init__(self, m):
             self._m = m
+            self._mask_radius = None
 
         def __call__(
             self, masked=True, mask_radius=None, mask_radius_crs="in", flat=False
@@ -1108,6 +1217,7 @@ class shapes(object):
             mask_radius : float, optional
                 the radius used for masking the delaunay-triangulation
                 (in units of the plot-crs)
+                The default is 4 times the estimated data-radius.
             mask_radius_crs : str, optional
                 The crs in which the radius is defined (either "in" or "out")
             flat : bool
@@ -1149,9 +1259,11 @@ class shapes(object):
         @property
         def mask_radius(self):
             if self.masked:
-                return shapes._get_radius(
-                    self._m, self._mask_radius, self.mask_radius_crs
-                )
+                r = shapes._get_radius(self._m, self._mask_radius, self.mask_radius_crs)
+                if self._mask_radius is None:
+                    return (i * 4 for i in r)
+                else:
+                    return r
             else:
                 return None
 
@@ -1170,7 +1282,7 @@ class shapes(object):
                 raise ImportError("'scipy' is required for 'delaunay_triangulation'!")
 
             # transform from crs to the plot_crs
-            t_in_plot = shapes.get_transformer(self._m.get_crs(crs), self._m.crs_plot)
+            t_in_plot = shapes._get_transformer(self._m.get_crs(crs), self._m.crs_plot)
 
             x0, y0 = t_in_plot.transform(x, y)
             datamask = np.isfinite(x0) & np.isfinite(y0)
@@ -1187,8 +1299,8 @@ class shapes(object):
 
                 if radius_crs == "in":
                     # use input-coordinates for evaluating the mask
-                    mx = self._m._props["xorig"].ravel()
-                    my = self._m._props["yorig"].ravel()
+                    mx = self._m._data_manager._current_data["xorig"].ravel()
+                    my = self._m._data_manager._current_data["yorig"].ravel()
                 elif radius_crs == "out":
                     # use projected coordinates for evaluating the mask
                     mx = x
@@ -1240,7 +1352,6 @@ class shapes(object):
                         pass
                     else:
                         color_and_array[key] = np.tile(val, 6)
-
                 coll = TriMesh(
                     tri,
                     # transOffset=self._m.ax.transData,
@@ -1520,9 +1631,9 @@ class shapes(object):
 
         def __repr__(self):
             try:
-                s = f"rectangles(radius={self.radius}, radius_crs={self.radius_crs})"
+                s = f"raster(radius={self.radius}, radius_crs={self.radius_crs})"
             except AttributeError:
-                s = "rectangles(radius, radius_crs)"
+                s = "raster(radius, radius_crs)"
 
             return s
 
@@ -1544,19 +1655,25 @@ class shapes(object):
                 rx = abs(dx[np.isfinite(dx)][0]) / 2
                 ry = abs(dy[np.isfinite(dy)][0]) / 2
 
-            self._radius = rx, ry
-            p = x, y
+            assert rx != 0 and ry != 0, (
+                "EOmaps: it seems the radius of the raster-data is 0... "
+                "Are the coordinates transposed?"
+            )
 
+            self._radius = rx, ry
+            # TODO switch to estimating the radius?
+            # rx, ry = self.radius
+            p = x, y
             in_crs = self._m.get_crs(crs)
 
             # transform corner-points
             in_crs = self._m.get_crs(crs)
             # transform from crs to the plot_crs
-            t = shapes.get_transformer(in_crs, self._m.crs_plot)
+            t = shapes._get_transformer(in_crs, self._m.crs_plot)
 
             # make sure we do not transform out of bounds (if possible)
             if in_crs.area_of_use is not None:
-                transformer = shapes.get_transformer(in_crs.geodetic_crs, in_crs)
+                transformer = shapes._get_transformer(in_crs.geodetic_crs, in_crs)
 
                 xmin, ymin, xmax, ymax = transformer.transform_bounds(
                     *in_crs.area_of_use.bounds
@@ -1582,7 +1699,10 @@ class shapes(object):
             px, py = t.transform(clipx(v[:, :, 0]), clipy(v[:, :, 1]))
             verts = np.stack((px, py), axis=2)
 
-            mask = np.logical_and(np.isfinite(px)[:-1, :-1], np.isfinite(py)[:-1, :-1])
+            # TODO is there a proper way to implement a mask here?
+            # (raster must always remain 2D...)
+            # mask = np.logical_and(np.isfinite(px)[:-1, :-1], np.isfinite(py)[:-1, :-1])
+            mask = np.full(px[:-1, :-1].shape, True, dtype=bool)
             return verts, mask
 
         def _get_polygon_coll(self, x, y, crs, **kwargs):
@@ -1593,8 +1713,8 @@ class shapes(object):
                 crs,
             )
 
-            # remember masked points
-            self._m._data_mask = mask
+            # TODO masking is skipped for now...
+            self._m._data_mask = None
             # don't use a mask here since we need the full 2D array
             color_and_array = shapes._get_colors_and_array(
                 kwargs, np.full_like(mask, True)
@@ -1629,6 +1749,11 @@ class shapes(object):
 
             return self._get_polygon_coll(x, y, crs, **kwargs)
 
+    @wraps(_scatter_points.__call__)
+    def scatter_points(self, *args, **kwargs):
+        shp = self._scatter_points(m=self._m)
+        return shp.__call__(*args, **kwargs)
+
     @wraps(_geod_circles.__call__)
     def geod_circles(self, *args, **kwargs):
         shp = self._geod_circles(m=self._m)
@@ -1652,6 +1777,10 @@ class shapes(object):
     @wraps(_voronoi_diagram.__call__)
     def voronoi_diagram(self, *args, **kwargs):
         shp = self._voronoi_diagram(m=self._m)
+        # increase radius margins for voronoi diagrams since
+        # outer points are otherwise always masked!
+        self._m._data_manager.set_margin_factors(20, 0.1)
+
         return shp.__call__(*args, **kwargs)
 
     @wraps(_delaunay_triangulation.__call__)
