@@ -87,6 +87,8 @@ class ColorBar:
         log=False,
         out_of_range_vals="clip",
         hist_kwargs=None,
+        label=None,
+        ylabel=None,
         **kwargs,
     ):
         """
@@ -205,6 +207,11 @@ class ColorBar:
         hist_kwargs : dict
             A dictionary with keyword-arguments passed to the creation of the histogram
             (e.g. passed to `plt.hist()` )
+        label : str, optional
+            The label used for the colorbar. The default is None
+        ylabel : str, optional
+            The label used for the y-axis of the colorbar. The default is None
+
         kwargs :
             All additional kwargs are passed to the creation of the colorbar
             (e.g. `plt.colorbar()`)
@@ -255,8 +262,6 @@ class ColorBar:
         else:
             self._hist_kwargs = copy.deepcopy(hist_kwargs)
 
-        self._depreciate_kwargs(kwargs)
-
         self._extend_frac = extend_frac
         self._orientation = orientation
         self._dynamic_shade_indicator = dynamic_shade_indicator
@@ -266,6 +271,7 @@ class ColorBar:
         self._log = log
         self._out_of_range_vals = out_of_range_vals
 
+        kwargs["label"] = label
         self._kwargs = copy.deepcopy(kwargs)
 
         self._coll = self._m.coll
@@ -287,11 +293,12 @@ class ColorBar:
 
         self._cid_redraw = None
 
-        self._refetch_bg = False
-
         self._set_data()
         self._setup_axes()
 
+        if ylabel is not None:
+            self.ax_cb_plot.set_ylabel(ylabel)
+
     def set_visible(self, vis):
         """
         Set the visibility of the colorbar
@@ -305,48 +312,6 @@ class ColorBar:
         for ax in self._axes:
             ax.set_visible(vis)
 
-    def _depreciate_kwargs(self, kwargs):
-        if "histbins" in kwargs:
-            self._hist_bins = kwargs.pop("histbins")
-            print(
-                "EOmaps: Colorbar argument 'histbins' is depreciated in EOmaps v5.0."
-                "Please use 'hist_bins' instead!"
-            )
-        if "density" in kwargs:
-            self._hist_kwargs["density"] = kwargs.pop("density")
-            print(
-                "EOmaps: Colorbar argument 'density' is depreciated in EOmaps v5.0."
-                "Please use `hist_kwargs=dict(density=...)` instead!"
-            )
-        if "histogram_size" in kwargs:
-            self._hist_size = kwargs.pop("histogram_size")
-            print(
-                "EOmaps: Colorbar argument 'histogram_size' is depreciated in EOmaps v5.0."
-                "Please use `hist_size` instead!"
-            )
-        if "add_extend_arrows" in kwargs:
-            kwargs.pop("add_extend_arrows")
-            print(
-                "EOmaps: Colorbar argument 'add_extend_arrows' is depreciated in EOmaps v5.0."
-                "Extend-arrows are added automatically if data out-of-range is found."
-            )
-
-        for key in ["top", "bottom", "left", "right"]:
-
-            margin = kwargs.pop("margin", None)
-            if margin is None:
-                margin = dict()
-
-            if key in kwargs:
-                margin[key] = kwargs.pop(key)
-                print(
-                    f"EOmaps: Colorbar argument '{key}' is depreciated in EOmaps v5.0."
-                    "Please use 'margin=dict({key}=...)' instead!"
-                )
-
-            if len(margin) > 0:
-                self._margin = margin
-
     def _default_cb_tick_formatter(self, x, pos, precision=None):
         """
         A formatter to format the tick-labels of the colorbar for encoded datasets.
@@ -386,57 +351,57 @@ class ColorBar:
         if size is not None:
             self._hist_size = size
 
-        # # evaluate axis padding due to extension-arrows
-        # if self._extend == "both":
-        #     d = self._extend_frac / (1 + 2 * self._extend_frac)
-        #     ex_min, ex_max = d, d
-        # elif self._extend == "min":
-        #     d = self._extend_frac / (1 + self._extend_frac)
-        #     ex_min, ex_max = d, 0.0
-        # elif self._extend == "max":
-        #     d = self._extend_frac / (1 + self._extend_frac)
-        #     ex_min, ex_max = 0.0, d
-        # else:
-        #     ex_min, ex_max = 0.0, 0.0
-
-        # no need to explicitly specify padding of extension arrows if axis-size
-        # is determined AFTER the colorbar is plotted!
-        ex_min, ex_max = 0.0, 0.0
-
-        if self._orientation == "horizontal":
-            s = 1 - self._hist_size
-            cbpos = mtransforms.Bbox(((0, 0), (1, s)))
-            histpos = mtransforms.Bbox(((ex_min, s), (1 - ex_max, 1)))
+        if self._inherit_position:
+            parent = self._get_parent_cb()
+            parent.set_hist_size(size)
         else:
-            s = 1 - self._hist_size
-            cbpos = mtransforms.Bbox(((1 - s, 0), (1, 1)))
-            histpos = mtransforms.Bbox(((0, ex_min), (1 - s, 1 - ex_max)))
+            # if the position is not inherited from a parent-axes, add margins
+            if self._margin is None:
+                if self._orientation == "horizontal":
+                    self._margin = dict(left=0.1, right=0.1, bottom=0.3, top=0.0)
+                else:
+                    self._margin = dict(left=0.0, right=0.3, bottom=0.1, top=0.1)
 
-        # colorbar axes
-        # NOTE: labels are used to hide those axes from the layout-manager!
-        if histpos.width > 1e-4 and histpos.height > 1e-4:
+            l, r = (self._margin.get(k, 0) for k in ["left", "right"])
+            b, t = (self._margin.get(k, 0) for k in ["bottom", "top"])
+            w, h = 1 - l - r, 1 - t - b
+
+            if self._orientation == "horizontal":
+                s = (1 - self._hist_size) * h
+                cbpos = (l, b, w, s)
+                histpos = (l, b + s, w, h - s)
+            else:
+                s = (1 - self._hist_size) * w
+                cbpos = (l + w - s, b, s, h)
+                histpos = (l, b, w - s, h)
+
+            self.ax_cb_plot.set_axes_locator(
+                _TransformedBoundsLocator(histpos, self._ax.transAxes)
+            )
+            self.ax_cb.set_axes_locator(
+                _TransformedBoundsLocator(cbpos, self._ax.transAxes)
+            )
+
+        # only show histogram if it is larger than 1 pixel
+        if self.ax_cb_plot.bbox.width > 1 and self.ax_cb_plot.bbox.height > 1:
             self.ax_cb_plot.set_visible(True)
         else:
-            self.ax_cb_plot.set_visible(False)
+            self.ax_cb_plot.set_visible(False)  # to avoid singular matrix errors
 
-        self.ax_cb_plot.set_axes_locator(
-            _TransformedBoundsLocator(histpos.bounds, self._ax.transAxes)
-        )
-
-        if cbpos.width > 1e-4 and cbpos.height > 1e-4:
+        if self.ax_cb.bbox.width > 1 and self.ax_cb.bbox.height > 1:
             self.ax_cb.set_visible(True)
             [i.set_visible(True) for i in self.ax_cb.patches]
             [i.set_visible(True) for i in self.ax_cb.collections]
         else:
+            self.ax_cb.set_visible(False)  # to avoid singular matrix errors
             [i.set_visible(False) for i in self.ax_cb.patches]
             [i.set_visible(False) for i in self.ax_cb.collections]
 
-        self.ax_cb.set_axes_locator(
-            _TransformedBoundsLocator(cbpos.bounds, self._ax.transAxes)
-        )
-
-        self._m.BM._refetch_layer(self._m.layer)
-        self._m.BM.update(artists=self._axes)
+        # don't update parent colorbar when child changes hist-size
+        if self._m.BM._bg_layer == self._m.layer:
+            self._m.BM.blit_artists(artists=self._axes)
+        else:
+            self._m.BM._refetch_layer(self._m.layer)
 
     def _identify_parent_cb(self):
         parent_cb = None
@@ -470,7 +435,6 @@ class ColorBar:
     def _setup_axes(self):
         horizontal = self._orientation == "horizontal"
         add_hist = self._hist_size > 0
-        add_margins = False
 
         # check if one of the parent colorbars has a colorbar, and if so,
         # use it to set the position of the colorbar.
@@ -478,12 +442,17 @@ class ColorBar:
             if self._parent_cb is not None:
 
                 try:
+                    parent_subplotspec = self._parent_cb._ax.get_subplotspec()
+                except AttributeError:
+                    parent_subplotspec = None
+
+                if parent_subplotspec is not None:
                     self._ax = self._m.f.add_subplot(
-                        self._parent_cb._ax.get_subplotspec(),
+                        parent_subplotspec,
                         label="cb",
                         zorder=9999,
                     )
-                except AttributeError:
+                else:
                     self._ax = self._m.f.add_axes(
                         self._parent_cb._ax.get_position(),
                         label="cb",
@@ -504,7 +473,6 @@ class ColorBar:
 
         if not self._inherit_position:
             if isinstance(self._pos, float):
-                add_margins = True
                 if horizontal:
                     gs = GridSpecFromSubplotSpec(
                         2,
@@ -534,7 +502,6 @@ class ColorBar:
                         zorder=9999,
                     )
             elif isinstance(self._pos, SubplotSpec):
-                add_margins = True
                 self._ax = self._m.f.add_subplot(
                     self._pos,
                     label="cb",
@@ -553,25 +520,35 @@ class ColorBar:
         # make all spines, labels etc. invisible for the base-axis
         self._ax.set_axis_off()
 
-        # NOTE: labels are used to hide those axes from the layout-manager!
         # colorbar axes
         self.ax_cb = self._ax.figure.add_axes(
-            self._ax.get_position(True),
+            self._ax.get_position(),
             label="EOmaps_cb",
             zorder=9998,
         )
         # histogram axes
         self.ax_cb_plot = self._ax.figure.add_axes(
-            self._ax.get_position(True),
+            self._ax.get_position(),
             label="EOmaps_cb_hist",
             zorder=9998,
         )
 
+        if self._inherit_position:
+            # in case the position is inherited, copy the locators from the parent!
+            self.ax_cb_plot.set_axes_locator(
+                _TransformedBoundsLocator(
+                    (0, 0, 1, 1), self._parent_cb.ax_cb_plot.transAxes
+                )
+            )
+            self.ax_cb.set_axes_locator(
+                _TransformedBoundsLocator((0, 0, 1, 1), self._parent_cb.ax_cb.transAxes)
+            )
+
         # join colorbar and histogram axes
         if horizontal:
-            self.ax_cb_plot.get_shared_x_axes().join(self.ax_cb_plot, self.ax_cb)
+            self.ax_cb_plot.sharex(self.ax_cb)
         else:
-            self.ax_cb_plot.get_shared_y_axes().join(self.ax_cb_plot, self.ax_cb)
+            self.ax_cb_plot.sharey(self.ax_cb)
 
         # keep the background of the plot-axis but remove the outer frame
         self.ax_cb_plot.spines["top"].set_visible(False)
@@ -595,36 +572,13 @@ class ColorBar:
         if not add_hist:
             self.ax_cb_plot.set_visible(False)
 
-        # add margins
-        if add_margins or self._margin is not None:
-            if self._margin is None:
-                if self._orientation == "horizontal":
-                    self._margin = dict(left=0.1, right=0.1, bottom=0.3, top=0.0)
-                else:
-                    self._margin = dict(left=0.0, right=0.3, bottom=0.1, top=0.1)
-
-            axpos = self._ax.get_position()
-            w, h = axpos.width, axpos.height
-            l, r = (self._margin.get(k, 0) * w for k in ["left", "right"])
-            b, t = (self._margin.get(k, 0) * h for k in ["bottom", "top"])
-
-            self._ax.set_position(
-                [
-                    axpos.x0 + l,
-                    axpos.y0 + b,
-                    axpos.width - l - r,
-                    axpos.height - b - t,
-                ]
-            )
-
         # add all axes as artists
         for a in self._axes:
             a.set_navigate(False)
             if a is not None:
                 self._m.BM.add_bg_artist(a, self._m.layer)
-
         # we need to re-draw since the background axis size has changed!
-        self._refetch_bg = True
+        self._m.BM._refetch_layer(self._m.layer)
 
     @property
     def _axes(self):
@@ -712,7 +666,7 @@ class ColorBar:
                 )
         else:
 
-            z_data = self._m._props["z_data"]
+            z_data = self._m._data_manager.z_data
             bins = self._m.classify_specs._bins
             cmap = self._m.classify_specs._cbcmap
             norm = self._m.classify_specs._norm
@@ -763,7 +717,6 @@ class ColorBar:
             self._redraw = True
 
     def _plot_colorbar(self):
-
         # plot the colorbar
         horizontal = self._orientation == "horizontal"
         n_cmap = plt.cm.ScalarMappable(cmap=self._cmap, norm=self._norm)
@@ -833,7 +786,7 @@ class ColorBar:
         horizontal = self._orientation == "horizontal"
         n_cmap = plt.cm.ScalarMappable(cmap=self._cmap, norm=self._norm)
 
-        if self._hist_size <= 0:
+        if self._hist_size <= 0.0001:
             return
 
         # plot the histogram
@@ -965,9 +918,6 @@ class ColorBar:
                 # self.ax_cb_plot.xaxis.set_major_locator(plt.MaxNLocator(5))
                 self.ax_cb_plot.set_xlim(None, 0)
 
-        if self._refetch_bg:
-            self._m.BM._refetch_layer(self._m.layer)
-
     def _redraw_colorbar(self, *args, **kwargs):
         self._set_data()
         if not self._redraw:
