@@ -424,30 +424,31 @@ class read_file:
                     + f"Available variables: {list(ncfile)}"
                 )
 
-            # 1D coordinates + 2D data expects transposed data!
-            transpose = False
-            if data.dims != x.dims or data.dims != x.dims:
-                if (len(data.dims) == 2 and len(x.dims) == 1 and len(y.dims) == 1) and (
-                    data.dims[0] == x.dims[0] and data.dims[1] == y.dims[0]
-                ):
-                    x, y = y, x
-                    transpose = True
-                elif (
-                    len(data.dims) == 2 and len(x.dims) == 1 and len(y.dims) == 1
-                ) and (data.dims[0] == y.dims[0] and data.dims[1] == x.dims[0]):
-                    transpose = True
-                else:
-                    raise AssertionError(
-                        "EOmaps: Invalid dimensions of data and coordinates!\n"
-                        + f"data: {data.dims},  x: {x.dims}, y: {y.dims}"
-                    )
+            check_shapes = (
+                (data.shape == (x.size, y.size))
+                or (data.shape == (y.size, x.size))
+                or (data.shape == x.shape and data.shape == y.shape)
+            )
+
+            if not check_shapes:
+                dstr = str([f"{i}: {j}" for i, j in zip(data.dims, data.shape)])
+                xstr = str([f"{i}: {j}" for i, j in zip(x.dims, x.shape)])
+                ystr = str([f"{i}: {j}" for i, j in zip(y.dims, y.shape)])
+                raise AssertionError(
+                    "EOmaps: Invalid dimensions of data and coordinates!\n"
+                    f"data: {dstr}\n"
+                    f"x   : {xstr}\n"
+                    f"y   : {ystr}\n"
+                )
+
+            if data.shape == (y.size, x.size) and len(x.shape) == 1:
+                data = data.values.T
+            else:
+                data = data.values
 
             # only use masked arrays if mask_and_scale is False!
             # (otherwise the mask is already applied as NaN's in the float-array)
             # Using masked-arrays ensures that we can deal with integers as well!
-
-            data = data.values
-
             if mask_and_scale is False:
                 encoding = dict(
                     scale_factor=getattr(usencfile[parameter], "scale_factor", 1),
@@ -462,7 +463,7 @@ class read_file:
 
             if set_data is not None:
                 set_data.set_data(
-                    data=data.T if transpose else data,
+                    data=data,
                     x=x.values,
                     y=y.values,
                     crs=data_crs,
@@ -471,7 +472,7 @@ class read_file:
                 )
             else:
                 return dict(
-                    data=data.T if transpose else data,
+                    data=data,
                     x=x.values,
                     y=y.values,
                     crs=data_crs,
@@ -567,6 +568,7 @@ def _from_file(
     parent=None,
     figsize=None,
     layer=None,
+    extent=None,
     **kwargs,
 ):
     """
@@ -617,6 +619,18 @@ def _from_file(
         to create a Maps-object for the dataset)
     figsize : tuple, optional
         The size of the figure. (Only relevant if parent is None!)
+    extent : tuple or string
+
+        - If a tuple is provided, it is used to set the plot-extent
+          before plotting via `m.set_extent(extent)`
+
+          - (x0, x1, y0, y1) : provide the extent in lat/lon
+          - ((x0, x1, y0, y1), crs) : provide the extent in the given crs
+
+        - If a string is provided, it is used to attempt to set the plot-extent
+          before plotting via `m.set_extent_to_location(extent)`
+
+        The default is None
     kwargs :
         Keyword-arguments passed to `m.plot_map()`
     Returns
@@ -675,6 +689,25 @@ def _from_file(
                 **{k: v for k, v in shape.items() if k != "shape"}
             )
 
+    if extent is not None:
+        if isinstance(extent, tuple):
+            if len(extent) == 2 and len(extent[0]) == 4:
+                m.set_extent(extent[0], crs=extent[1])
+            elif len(extent) == 4:
+                m.set_extent(extent)
+            else:
+                print(
+                    "EOmaps: unable to identify the provided extent"
+                    f"{extent}... defaulting to global"
+                )
+        elif isinstance(extent, str):
+            m.set_extent_to_location(extent)
+        else:
+            print(
+                "EOmaps: unable to identify the provided extent"
+                f"{extent}... defaulting to global"
+            )
+
     m.plot_map(**kwargs)
     return m
 
@@ -804,6 +837,20 @@ class from_file:
             values for callbacks and colorbars, even if `mask_and_scale=False`!
 
             The default is False.
+        extent : tuple or string
+            Set the extent of the map prior to plotting
+            (can provide great speedups if only a subset of the dataset is shown!)
+
+            - If a tuple is provided, it is used to set the plot-extent
+              before plotting via `m.set_extent(extent)`
+
+              - (x0, x1, y0, y1) : provide the extent in lat/lon
+              - ((x0, x1, y0, y1), crs) : provide the extent in the given crs
+
+            - If a string is provided, it is used to attempt to set the plot-extent
+              before plotting via `m.set_extent_to_location(extent)`
+
+            The default is None
         kwargs :
             Keyword-arguments passed to `m.plot_map()`
 
@@ -968,6 +1015,20 @@ class from_file:
             values for callbacks and colorbars, even if `mask_and_scale=False`!
 
             The default is False.
+        extent : tuple or string
+            Set the extent of the map prior to plotting
+            (can provide great speedups if only a subset of the dataset is shown!)
+
+            - If a tuple is provided, it is used to set the plot-extent
+              before plotting via `m.set_extent(extent)`
+
+              - (x0, x1, y0, y1) : provide the extent in lat/lon
+              - ((x0, x1, y0, y1), crs) : provide the extent in the given crs
+
+            - If a string is provided, it is used to attempt to set the plot-extent
+              before plotting via `m.set_extent_to_location(extent)`
+
+            The default is None
         kwargs :
             Keyword-arguments passed to `m.plot_map()`
 
@@ -1103,6 +1164,21 @@ class from_file:
             Additional kwargs passed to pandas.read_file()
             (e.g. to set index_col etc.)
             The default is None
+        extent : tuple or string
+            Set the extent of the map prior to plotting
+            (can provide great speedups if only a subset of the dataset is shown!)
+
+            - If a tuple is provided, it is used to set the plot-extent
+              before plotting via `m.set_extent(extent)`
+
+              - (x0, x1, y0, y1) : provide the extent in lat/lon
+              - ((x0, x1, y0, y1), crs) : provide the extent in the given crs
+
+            - If a string is provided, it is used to attempt to set the plot-extent
+              before plotting via `m.set_extent_to_location(extent)`
+
+            The default is None
+
         kwargs :
             Keyword-arguments passed to `m.plot_map()`
 
