@@ -14,6 +14,8 @@ import matplotlib.transforms as transforms
 
 
 class ScaleBar:
+    """Base class for EOmaps scalebars."""
+
     def __init__(
         self,
         m,
@@ -24,9 +26,11 @@ class ScaleBar:
         scale_props=None,
         patch_props=None,
         label_props=None,
+        layer=None,
     ):
         """
         Add a scalebar to the map.
+
         The scalebar represents a ruler in units of meters whose direction
         follows geodesic lines.
 
@@ -124,7 +128,29 @@ class ScaleBar:
 
             The default is:
                 >>> dict(scale=1, offset=1, rotation=0, every=2)
+        layer : str, optional
+            The layer at which the scalebar should be visible.
+            If None, the layer of the Maps-object used to create the scalebar is used.
+            The default is None.
+
         """
+        self._m = m
+
+        if layer is None:
+            layer = self._m.layer
+        self.layer = layer
+
+        self._scale_props = dict(scale=None)
+        self._label_props = dict()
+        self._patch_props = dict()
+        self._patch_offsets = (1, 1, 1, 1)
+
+        self._font_kwargs = dict()
+        self._fontkeys = ("family", "style", "variant", "stretch", "weight")
+
+        # apply preset styling (so that any additional properties are applied on top
+        # of the preset)
+        self._apply_preset(preset)
 
         if scale is None:
             self._autoscale = autoscale_fraction
@@ -133,7 +159,11 @@ class ScaleBar:
 
         self._auto_position = auto_position
 
-        self._m = m
+        self.set_scale_props(scale=scale, **(scale_props if scale_props else {}))
+        # set the label properties
+        self.set_label_props(**(label_props if label_props else {}))
+        # set the patch properties
+        self.set_patch_props(**(patch_props if patch_props else {}))
 
         # number of intermediate points for evaluating the curvature
         self._interm_pts = 20
@@ -141,20 +171,6 @@ class ScaleBar:
         self._cb_offset_interval = 0.05
         self._cb_rotate_inverval = 1
 
-        self._fontkeys = ("family", "style", "variant", "stretch", "weight")
-        self._font_kwargs = dict()
-
-        self._scale_props = dict(scale=scale, n=10, width=5, colors=("k", "w"))
-        self._patch_props = dict(fc=".75", ec="k", lw=1, ls="-")
-        self._patch_offsets = (1, 1, 1, 1)
-        self._label_props = dict(scale=2, rotation=0, every=2, offset=1, color="k")
-
-        self.set_scale_props(scale=scale, **(scale_props if scale_props else {}))
-        # set the label properties
-        self.set_label_props(**(label_props if label_props else {}))
-        # set the patch properties
-        self.set_patch_props(**(patch_props if patch_props else {}))
-
         # geod from plot_crs
         self._geod = self._m.crs_plot.get_geod()
         # Transformer from lon/lat to the plot_crs
@@ -171,24 +187,51 @@ class ScaleBar:
         )
 
         self._artists = OrderedDict(patch=None, scale=None)
-
-        self.preset = preset
-
         self._picker_name = None
 
-    def apply_preset(self, preset):
-        if preset == "bw":
-            self.set_scale_props(n=10, width=4, colors=("k", "w"))
-            self.set_patch_props(fc="none", ec="none", offsets=(1, 1.6, 1, 1))
-            self.set_label_props(
-                scale=1.5, offset=0.5, every=2, weight="bold", family="Courier New"
-            )
+    def _get_preset_props(self, preset):
+        scale_props = dict(n=10, width=5, colors=("k", "w"))
+        patch_props = dict(fc=".75", ec="k", lw=1, ls="-")
+        label_props = dict(scale=2, offset=1, every=2, rotation=0, color="k")
 
-            self._estimate_scale()
-            self.set_position()
+        if preset == "bw":
+            scale_props.update(dict(n=10, width=4, colors=("k", "w")))
+            patch_props.update(dict(fc="none", ec="none"))
+            label_props.update(
+                dict(
+                    scale=1.5,
+                    offset=0.5,
+                    every=2,
+                    weight="bold",
+                    family="Courier New",
+                )
+            )
+        return scale_props, patch_props, label_props
+
+    def _apply_preset(self, preset):
+        self.preset = preset
+
+        scale_props, patch_props, label_props = self._get_preset_props(preset)
+        self.set_scale_props(**scale_props)
+        self.set_patch_props(**patch_props)
+        self.set_label_props(**label_props)
+
+    def apply_preset(self, preset):
+        """
+        Apply a style-preset to the Scalebar.
+
+        Parameters
+        ----------
+        preset : str
+            The name of the preset.
+
+        """
+        self._apply_preset(preset)
+        self._estimate_scale()
+        self.set_position()
 
     @staticmethod
-    def round_to_n(x, n=0):
+    def _round_to_n(x, n=0):
         # round to n significant digits
         # 1234 -> 1000
         # 0.01234 -> 0.1
@@ -223,7 +266,7 @@ class ScaleBar:
 
             scale = dist / self._scale_props["n"]
             # round to 1 significant digit
-            scale = self.round_to_n(scale)
+            scale = self._round_to_n(scale)
 
             self._scale_props["scale"] = scale
             return scale
@@ -249,15 +292,13 @@ class ScaleBar:
         return lon, lat
 
     def auto_position(self, pos):
+        """Move the scalebar to the desired position and apply auto-scaling."""
         lon, lat = self._get_autopos(pos)
         self.set_position(lon, lat, self._azim)
 
     @property
     def cb_rotate_interval(self):
-        """
-        Get/set the interval for the rotation when using the <+> and <->
-        keys on the keyboard to rotate the scalebar
-        """
+        """Get/set the rotation interval when rotating the scalebar with +/- keys."""
         return self._cb_rotate_inverval
 
     @cb_rotate_interval.setter
@@ -267,8 +308,10 @@ class ScaleBar:
     @property
     def cb_offset_interval(self):
         """
-        Get/set the interval for the text-offset when using the <alt> + <+>/<->
-        keyboard-shortcut to set the offset for the scalebar-labels.
+        Get/set the interval for changing the text-offset with keyboard-shortcuts.
+
+        e.g.: when using the <alt> + <+>/<-> keyboard-shortcut to set the offset for
+        the scalebar-labels.
         """
         return self._cb_offset_interval
 
@@ -278,7 +321,7 @@ class ScaleBar:
 
     def set_scale_props(self, scale=None, n=None, width=None, colors=None):
         """
-        Set the properties of the scalebar (and update the plot accordingly)
+        Set the properties of the scalebar (and update the plot accordingly).
 
         Parameters
         ----------
@@ -308,11 +351,11 @@ class ScaleBar:
 
         if hasattr(self, "_lon") and hasattr(self, "_lat"):
             self.set_position()
-            self._m.BM.update(blit=False)
+            self._m.BM.blit_artists(self._artists.values())
 
     def set_patch_props(self, offsets=None, **kwargs):
         """
-        Set the properties of the frame (and update the plot accordingly)
+        Set the properties of the frame (and update the plot accordingly).
 
         Parameters
         ----------
@@ -338,21 +381,22 @@ class ScaleBar:
             ["lw", "linewidth"],
             ["ls", "linestyle"],
         ]:
-            self._patch_props[key] = kwargs.pop(
-                key, kwargs.pop(synonym, self._patch_props[key])
-            )
+            if key in self._patch_props:
+                self._patch_props[key] = kwargs.pop(
+                    key, kwargs.pop(synonym, self._patch_props[key])
+                )
 
         self._patch_props.update(kwargs)
 
         if hasattr(self, "_lon") and hasattr(self, "_lat"):
             self.set_position()
-            self._m.BM.update(blit=False)
+            self._m.BM.blit_artists(self._artists.values())
 
     def set_label_props(
         self, scale=None, rotation=None, every=None, offset=None, color=None, **kwargs
     ):
         """
-        Set the properties of the labels (and update the plot accordingly)
+        Set the properties of the labels (and update the plot accordingly).
 
         Parameters
         ----------
@@ -379,7 +423,6 @@ class ScaleBar:
                 >>> dict(family="Helvetica", style="italic").
 
         """
-
         if scale is not None:
             self._label_props["scale"] = scale
         if rotation is not None:
@@ -401,7 +444,7 @@ class ScaleBar:
 
         if hasattr(self, "_lon") and hasattr(self, "_lat"):
             self.set_position()
-            self._m.BM.update(blit=False)
+            self._m.BM.blit_artists(self._artists.values())
 
     def _get_base_pts(self, lon, lat, azim, npts=None):
         if npts is None:
@@ -463,6 +506,7 @@ class ScaleBar:
 
     def _get_d(self):
         # the base length used to define the size of the scalebar
+
         # get the position in figure coordinates
         x, y = self._m.ax.transData.transform(
             self._t_plot.transform(self._lon, self._lat)
@@ -476,16 +520,15 @@ class ScaleBar:
         return np.abs(xb[1] - yb[1])
 
     def _get_patch_verts(self, pts, lon, lat, ang, d):
-        ot = 0.75 * d * self._patch_offsets[0]  # top offset
-        # ob = 2.5 * d * self._patch_offsets[1]  # bottom offset
-        ob = (2.5 * d + self._maxw) * self._patch_offsets[1]  # bottom offset
+        # top bottom left right referrs to a horizontally oriented colorbar!
+        ot = d * self._patch_offsets[0]
+        ob = self._maxw + d * (self._label_props["offset"] + self._patch_offsets[1])
+        o_l = d * self._patch_offsets[2]
+        o_r = d * self._patch_offsets[3]
 
-        ob = ((self._label_props["offset"] + 2) * d + self._maxw) * self._patch_offsets[
-            1
-        ]  # bottom offset
-
-        o_l = 0.5 * d * self._patch_offsets[2]  # left offset
-        o_r = 1.5 * d * self._patch_offsets[3]  # right offset
+        # in case the top scale has a label, add a margin to encompass the text!
+        if len(pts) % self._label_props["every"] == 0:
+            o_r += self._top_h * 1.5
 
         dxy = np.gradient(pts.reshape((-1, 2)), axis=0)
         alpha = np.arctan2(dxy[:, 1], -dxy[:, 0])
@@ -500,6 +543,8 @@ class ScaleBar:
         ptop[-1] -= (o_r * np.cos(alpha[-1]), -o_r * np.sin(alpha[-1]))
         pbottom[-1] -= (o_r * np.cos(alpha[-1]), -o_r * np.sin(alpha[-1]))
 
+        # TODO check how to deal with invalid vertices (e.g. self-intersections)
+
         return np.vstack([ptop, pbottom[::-1]])
 
     def _get_ang(self, p0, p1):
@@ -516,8 +561,8 @@ class ScaleBar:
     def _get_txt_coords(self, lon, lat, d, ang):
         # get the base point for the text
         xt, yt = self._t_plot.transform(lon, lat)
-        xt = xt - d * self._label_props["offset"] * np.sin(ang) / 2
-        yt = yt + d * self._label_props["offset"] * np.cos(ang) / 2
+        xt = xt - d * self._label_props["offset"] * np.sin(ang)
+        yt = yt + d * self._label_props["offset"] * np.cos(ang)
         return xt, yt
 
     from functools import lru_cache
@@ -540,16 +585,27 @@ class ScaleBar:
             # if the object is within the canvas!
             try:
                 # get the widths of the text patches in data-coordinates
-                bbox = val.get_tightbbox(self._m.f.canvas.get_renderer())
+                bbox = val.get_window_extent(self._m.f.canvas.get_renderer())
                 bbox = bbox.transformed(self._m.ax.transData.inverted())
-
                 # use the max to account for rotated text objects
                 w = max(bbox.width, bbox.height)
                 if w > _maxw:
                     _maxw = w
             except Exception:
                 pass
+
+        _top_h = 0
+        try:
+            _top_label = next(i for i in sorted(self._artists) if i.startswith("text_"))
+            val = self._artists[_top_label]
+            bbox = val.get_window_extent(self._m.f.canvas.get_renderer())
+            bbox = bbox.transformed(self._m.ax.transData.inverted())
+            _top_h = min(bbox.width, bbox.height)
+        except Exception:
+            pass
+
         self._maxw = _maxw
+        self._top_h = _top_h
 
     def _set_minitxt(self, d, pts):
         angs = np.arctan2(*np.array([p[0] - p[-1] for p in pts]).T[::-1])
@@ -566,9 +622,7 @@ class ScaleBar:
             else:
                 txt = self._get_txt(i)
 
-            xy = self._get_txt_coords(
-                lon, lat, self._label_props["scale"] * d * 1.5, ang
-            )
+            xy = self._get_txt_coords(lon, lat, d, ang)
             tp = TextPath(
                 xy, txt, size=self._label_props["scale"] * d / 2, prop=self._font_props
             )
@@ -584,7 +638,7 @@ class ScaleBar:
             )
 
             self._artists[f"text_{i}"].set_zorder(1)
-            self._m.BM.add_artist(self._artists[f"text_{i}"])
+            self._m.BM.add_artist(self._artists[f"text_{i}"], layer=self.layer)
 
     def _redraw_minitxt(self):
         # don't redraw if we haven't drawn anything yet
@@ -617,9 +671,7 @@ class ScaleBar:
             else:
                 txt = self._get_txt(i)
 
-            xy = self._get_txt_coords(
-                lon, lat, self._label_props["scale"] * d * 1.5, ang
-            )
+            xy = self._get_txt_coords(lon, lat, d, ang)
 
             tp = PathPatch(
                 TextPath(
@@ -691,25 +743,21 @@ class ScaleBar:
         coll.set_linewidth(self._scale_props["width"])
         self._artists["scale"] = self._m.ax.add_collection(coll, autolim=False)
 
-        # apply preset
-        self.apply_preset(self.preset)
-
         # -------------- make all artists animated
         self._artists["scale"].set_zorder(1)
         self._artists["patch"].set_zorder(0)
 
-        self._m.BM.add_artist(self._artists["scale"])
+        self._m.BM.add_artist(self._artists["scale"], layer=self.layer)
         # self._m.BM.add_artist(self._artists["text"])
-        self._m.BM.add_artist(self._artists["patch"])
-
-        self._m.BM.update(artists=self._artists.values())
+        self._m.BM.add_artist(self._artists["patch"], layer=self.layer)
 
+        self._m.BM.blit_artists(self._artists.values())
         # make sure to update the artists on zoom
         self._decorate_zooms()
 
     def set_position(self, lon=None, lat=None, azim=None, update=False):
         """
-        Sset the position of the colorbar
+        Set the position of the colorbar.
 
         Parameters
         ----------
@@ -723,8 +771,8 @@ class ScaleBar:
         update : bool
             Indicator if the plot should be immediately updated (True) or at the next
             event (False). The default is False.
-        """
 
+        """
         if lon is None:
             lon = self._lon
         if lat is None:
@@ -752,6 +800,12 @@ class ScaleBar:
         self._artists["scale"].set_colors(colors)
 
         verts = self._get_patch_verts(pts, lon, lat, ang, d)
+
+        # TODO check how to deal with invalid vertices!!
+        # print(np.all(np.isfinite(self._m._transf_plot_to_lonlat.transform(*verts.T))))
+
+        # verts = np.ma.masked_invalid(verts)
+
         self._artists["patch"].set_verts([verts])
         self._artists["patch"].update(self._patch_props)
 
@@ -766,7 +820,7 @@ class ScaleBar:
 
     def _make_pickable(self):
         """
-        Add callbacks to adjust the scalebar position manually
+        Add callbacks to adjust the scalebar position manually.
 
             - <LEFT>-click on the scalebar with the mouse to pick it up
                 - hold down <LEFT> to drag the scalebar
@@ -774,6 +828,7 @@ class ScaleBar:
             - <RIGHT>-click on the scalebar to detach the callbacks again
               (e.g. make it non-interactive)
             - use <ARROW-keys> to set the size of the patch
+
         """
 
         def scb_move(s, pos, **kwargs):
@@ -953,8 +1008,7 @@ class ScaleBar:
 
     def get_position(self):
         """
-        Return the current position (and orientation) of the scalebar
-        (e.g. to obtain the position after manual re-positioning)
+        Return the current position (and orientation) of the scalebar.
 
         Returns
         -------
@@ -964,9 +1018,7 @@ class ScaleBar:
         return [self._lon, self._lat, self._azim]
 
     def remove(self):
-        """
-        Remove the scalebar from the map
-        """
+        """Remove the scalebar from the map."""
         self._remove_callbacks()
         for a in self._artists.values():
             self._m.BM.remove_artist(a)
@@ -976,6 +1028,8 @@ class ScaleBar:
 
 
 class Compass:
+    """Base class for EOmaps compass objects."""
+
     def __init__(self, m):
         self._m = m
 
@@ -985,8 +1039,89 @@ class Compass:
         self._txt = "N"
 
     def __call__(
-        self, pos=None, scale=10, style="north arrow", patch="w", txt="N", pickable=True
+        self,
+        pos=None,
+        pos_transform="axes",
+        scale=10,
+        style="compass",
+        patch=None,
+        txt="N",
+        pickable=True,
+        layer=None,
+        ignore_invalid_angles=False,
     ):
+        """
+        Add a "compass" or "north-arrow" to the map.
+
+        Note
+        ----
+        You can use the mouse to pick the compass and move it anywhere on the map.
+        (the directions are dynamically updated if you pan/zoom or pick the compass)
+
+        - If you press the "delete" key while clicking on the compass, it is removed.
+          (same as calling `compass.remove()`)
+        - If you press the "d" key while clicking on the compass, it will be
+          disconnected from pick-events (same as calling `compass.set_pickable(False)`)
+
+
+        Parameters
+        ----------
+        pos : tuple or None, optional
+            The initial position of the compass with respect to the transformation
+            defined as "pos_transform".
+            Note that you can also move the compass with the mouse!
+        pos_transform : string, optional
+            Indicator in what coordinate-system the initial position is provided.
+
+            - "axes": relative axis-coordinates in the range (0-1)
+            - "lonlat": coordinates provided as (longitude, latitude)
+            - "plot_crs": coordinates provided in the crs used for plotting.
+
+            The default is "axes".
+        scale : float, optional
+            A scale-factor for the size of the compass. The default is 10.
+        style : str, optional
+
+            - "north arrow" : draw only a north-arrow
+            - "compass": draw a compass with arrows in all 4 directions
+
+            The default is "compass".
+        patch : False, str or tuple, optional
+            The color of the background-patch.
+            (can be any color specification supported by matplotlib)
+            The default is "w".
+        txt : str, optional
+            Indicator which directions should be indicated.
+            - "NESW" : add letters for all 4 directions
+            - "NE" : add only letters for North and East (same for other combinations)
+            - None : don't add any letters
+            The default is "N".
+        pickable : bool, optional
+            Indicator if the compass should be static (False) or if it can be dragged
+            with the mouse (True). The default is True
+        layer : str, optional
+            The layer to put the compass on. The default is "all".
+        ignore_invalid_angles : bool, optional
+            - If True the compass will always (silently) use the last valid rotation-angle
+              in case the correct angle could not be determined.
+            - If False, a warning will be issued in case the angle could
+              not be determined, and a red border will be drawn around the compass to
+              indicate that it might not point in the right direction.
+
+            The default is False
+
+        Returns
+        -------
+        compass : eomaps.Compass
+            A compass-object that can be used to manually adjust the style and position
+            of the compass or remove it from the map.
+
+        """
+        if layer is None:
+            layer = self._m.layer
+        self.layer = layer
+
+        self._ignore_invalid_angles = ignore_invalid_angles
         self._m.BM.update()
 
         ax2data = self._m.ax.transAxes + self._m.ax.transData.inverted()
@@ -994,16 +1129,32 @@ class Compass:
         if pos is None:
             pos = ax2data.transform((0.5, 0.5))
         else:
-            pos = ax2data.transform(pos)
+            if pos_transform == "axes":
+                pos = ax2data.transform(pos)
+            elif pos_transform == "lonlat":
+                pos = self._m._transf_lonlat_to_plot.transform(*pos)
+            elif pos_transform == "plot_crs":
+                pass
+            else:
+                raise TypeError(
+                    f"EOmaps: {pos_transform} is not a valid 'pos_transform'."
+                    "Use one of ('axes', 'lonlat', 'plot_crs')"
+                )
 
         self._style = style
         self._patch = patch
         self._txt = txt
         self._scale = scale
+        # remember the dpi at the time the compass was initialized
+        self._init_dpi = self._m.f.dpi
+
+        self._ang = 0
+        # remember last used rotation angle for out-of-axes compass
+        self._last_ang = 999
 
         self._artist = self._get_artist(pos)
         self._m.ax.add_artist(self._artist)
-        self._m.BM.add_artist(self._artist)
+        self._m.BM.add_artist(self._artist, layer=self.layer)
 
         self.set_position(pos)
 
@@ -1016,13 +1167,11 @@ class Compass:
         self._cids = [
             self._canvas.mpl_connect("pick_event", self._on_pick),
             self._canvas.mpl_connect("button_release_event", self._on_release),
-            self._canvas.mpl_connect("resize_event", self._on_resize),
+            self._canvas.mpl_connect("scroll_event", self._on_scroll),
         ]
 
-        self._add_zoom_callbacks()
-
-    def _on_resize(self, event):
-        self._update_offset(*self._pos)
+        if self._update_offset not in self._m.BM._before_fetch_bg_actions:
+            self._m.BM._before_fetch_bg_actions.append(self._update_offset)
 
     def _get_artist(self, pos):
         if self._style == "north arrow":
@@ -1098,20 +1247,19 @@ class Compass:
 
         return art
 
-    def _update_offset(self, x, y):
+    def _update_offset(self, x=None, y=None, *args, **kwargs):
         # reset to the center of the axis if both are None
         try:
-            ax2data = self._m.ax.transAxes + self._m.ax.transData.inverted()
-
             if x is None or y is None:
                 try:
-                    x, y = ax2data.transform(
-                        self._ax2data.inverted().transform(self._pos)
-                    )
+                    self.set_position(self._pos)
+                    return
                 except Exception:
                     x, y = 0.9, 0.1
+                    self.set_position((x, y), "axis")
+                    return
 
-            self.set_position((x, y))
+            self.set_position((x, y), "data")
         except Exception:
             pass
 
@@ -1126,29 +1274,88 @@ class Compass:
             print("EOmaps: could not add scalebar at the desired location")
             return
 
+        if np.isnan(ang):
+            if not self._ignore_invalid_angles:
+                if self._last_ang != self._ang:
+                    print(
+                        "EOmaps: Compass rotation-angle could not be determined! "
+                        f"... using last found angle: {np.rad2deg(self._ang):.2f}"
+                    )
+                    patch = self._artist.get_children()[0]
+                    self._patch_ec = patch.get_edgecolor()
+                    patch.set_edgecolor("r")
+
+                self._last_ang = self._ang
+            else:
+                if hasattr(self, "_patch_ec"):
+                    self._artist.get_children()[0].set_edgecolor(self._patch_ec)
+                    del self._patch_ec
+                self._last_ang = 9999
+
+            ang = self._ang
+        else:
+            if hasattr(self, "_patch_ec"):
+                self._artist.get_children()[0].set_edgecolor(self._patch_ec)
+                del self._patch_ec
+
+            self._last_ang = 9999
+
+        self._ang = ang
         r = transforms.Affine2D().rotate(ang)
-        s = transforms.Affine2D().scale(self._scale)
+        # apply the scale-factor with respect to the current figure dpi to keep the
+        # relative size of the north-arrow on dpi-changes!
+        s = transforms.Affine2D().scale(self._scale * self._m.f.dpi / self._init_dpi)
         t = transforms.Affine2D().translate(*self._m.ax.transData.transform(pos))
         trans = r + s + t
         return trans
 
     def _on_motion(self, evt):
+        if not self._layer_visible:
+            return
+
         if self._check_still_parented() and self._got_artist:
             x, y = evt.xdata, evt.ydata
 
-            if x is None or y is None:
-                # continue values outside of the crs-domain
+            # transform values if axes is put outside the figure
+            if evt.inaxes is None:
                 x, y = self._m.ax.transData.inverted().transform((evt.x, evt.y))
+            elif evt.inaxes != self._m.ax:
+                # don't allow moving the compass on top of another axes
+                # (somehow pick-events do not fire if compass is in another axes)
+                # TODO check this!
+                return
+
+            # continue values outside of the crs-domain
+            if x is None or y is None:
+                x, y = self._m.ax.transData.inverted().transform((evt.x, evt.y))
+
             self._update_offset(x, y)
-            self._m.BM._draw_animated(artists=[self._artist])
+            self._m.BM.blit_artists(artists=[self._artist])
+
+    def _on_scroll(self, event):
+        if not self._layer_visible:
+            return
+
+        if self._check_still_parented() and self._got_artist:
+            self.set_scale(max(1, self._scale + event.step))
+            self._m.BM.blit_artists(artists=[self._artist])
 
     def _on_pick(self, evt):
+        if not self._layer_visible:
+            return
+
+        if evt.mouseevent.button != 1:
+            return
+
         if self._check_still_parented() and evt.artist == self._artist:
             self._got_artist = True
             self._c1 = self._canvas.mpl_connect("motion_notify_event", self._on_motion)
             self._c2 = self._canvas.mpl_connect("key_press_event", self._on_keypress)
 
     def _on_keypress(self, event):
+        if not self._layer_visible:
+            return
+
         if event.key == "delete":
             self.remove()
         if event.key == "d":
@@ -1156,6 +1363,9 @@ class Compass:
             self.set_pickable(False)
 
     def _on_release(self, event):
+        if not self._layer_visible:
+            return
+
         if self._check_still_parented() and self._got_artist:
             self._finalize_offset()
             self._got_artist = False
@@ -1172,6 +1382,7 @@ class Compass:
                 pass
             else:
                 self._canvas.mpl_disconnect(c2)
+            self._m.BM.update()
 
     def _check_still_parented(self):
         if self._artist.figure is None:
@@ -1180,13 +1391,19 @@ class Compass:
         else:
             return True
 
+    @property
+    def _layer_visible(self):
+        return self.layer == "all" or (
+            self.layer in (*self._m.BM.bg_layer.split("|"), self._m.BM.bg_layer)
+        )
+
     def _disconnect(self):
         """Disconnect the callbacks."""
         for cid in self._cids:
             self._canvas.mpl_disconnect(cid)
 
-        for cid in self._ax_cids:
-            self._m.ax.callbacks.disconnect(cid)
+        if self._update_offset in self._m.BM._before_fetch_bg_actions:
+            self._m.BM._before_fetch_bg_actions.append(self._update_offset)
 
         try:
             c1 = self._c1
@@ -1205,25 +1422,6 @@ class Compass:
     def _finalize_offset(self):
         pass
 
-    def _add_zoom_callbacks(self):
-        self._ax_cids = set()
-
-        self._ax_cids.add(
-            self._m.ax.callbacks.connect(
-                "xlim_changed", lambda *args: self._update_offset(None, None)
-            )
-        )
-        self._ax_cids.add(
-            self._m.ax.callbacks.connect(
-                "ylim_changed", lambda *args: self._update_offset(None, None)
-            )
-        )
-        self._ax_cids.add(
-            self._m.f.canvas.mpl_connect(
-                "resize_event", lambda *args: self._update_offset(None, None)
-            )
-        )
-
     def remove(self):
         """
         Remove the compass from the map.
@@ -1250,8 +1448,8 @@ class Compass:
             - False or "none": Make the background-patch invisible.
         linewidth: float
             The linewidth of the patch.
-        """
 
+        """
         if facecolor is False:
             facecolor = "none"
         if edgecolor is False:
@@ -1269,14 +1467,14 @@ class Compass:
 
     def set_scale(self, scale):
         """
-        Set the size of the  compass. (The default is 10)
+        Set the size scale-factor of the compass. (The default is 10).
 
         Parameters
         ----------
         s : float
             The size of the compass.
-        """
 
+        """
         self._scale = scale
         self._update_offset(*self._pos)
 
@@ -1311,13 +1509,71 @@ class Compass:
 
             The default is "data".
         """
-        self._ax2data = self._m.ax.transAxes + self._m.ax.transData.inverted()
-
         if coords == "axis":
+            self._ax2data = self._m.ax.transAxes + self._m.ax.transData.inverted()
             pos = self._ax2data.transform(pos)
 
         trans = self._get_transform(pos)
         for c in self._artist.get_children():
             c.set_transform(trans)
-
         self._pos = pos
+
+    def get_position(self, coords="data"):
+        """
+        Return the current position of the compass.
+
+        Parameters
+        ----------
+        coords : str, optional
+            Define what coordinates are returned
+
+            - "data" : coordinates in the plot-crs
+            - "axis": relative [0-1] coordinates with respect to the
+              axis (e.g. (0, 0) = lower left corner, (1, 1) = upper right corner)
+
+            The default is "data".
+
+        Returns
+        -------
+        pos
+            a tuple (x, y) representing the current location of the compass.
+
+        """
+        self._ax2data = self._m.ax.transAxes + self._m.ax.transData.inverted()
+
+        if coords == "axis":
+            return self._ax2data.inverted().transform(self._pos)
+        elif coords == "data":
+            return self._pos
+        else:
+            raise TypeError("EOmaps: 'coords' must be one of ['data', 'axis']!")
+
+    def get_scale(self):
+        """
+        Return the current size scale-factor of the compass.
+
+        Returns
+        -------
+        s : float
+            The size of the compass.
+
+        """
+        return self._scale
+
+    def set_ignore_invalid_angles(self, val):
+        """
+        Set how to deal with invalid rotation-angles.
+
+        - If True the compass will always (silently) use the last valid rotation-angle
+          in case the correct angle could not be determined.
+        - If False (the default), a warning will be issued in case the angle could
+          not be determined, and a red border will be drawn around the compass to
+          indicate that it might not point in the right direction.
+
+        Parameters
+        ----------
+        val : bool
+            ignore invalid rotation angles.
+        """
+        self._ignore_invalid_angles = val
+        self.set_position(self._pos)
